@@ -44,6 +44,16 @@ static int misc_check_thread(thread_t *);
 static int misc_check_child_thread(thread_t *);
 static int misc_check_child_timeout_thread(thread_t *);
 
+static bool script_user_set;
+static misc_checker_t *new_misck_checker;
+static bool have_dynamic_misc_checker;
+
+void
+clear_dynamic_misc_check_flag(void)
+{
+	have_dynamic_misc_checker = false;
+}
+
 /* Configuration stream handling */
 static void
 free_misc_check(void *data)
@@ -58,61 +68,125 @@ free_misc_check(void *data)
 static void
 dump_misc_check(void *data)
 {
-	misc_checker_t *misck_checker = CHECKER_DATA(data);
+	checker_t *checker = data;
+	misc_checker_t *misck_checker = checker->data;
+
 	log_message(LOG_INFO, "   Keepalive method = MISC_CHECK");
 	log_message(LOG_INFO, "   script = %s", misck_checker->path);
 	log_message(LOG_INFO, "   timeout = %lu", misck_checker->timeout/TIMER_HZ);
 	log_message(LOG_INFO, "   dynamic = %s", misck_checker->dynamic ? "YES" : "NO");
 	log_message(LOG_INFO, "   uid:gid = %d:%d", misck_checker->uid, misck_checker->gid);
 	log_message(LOG_INFO, "   insecure = %s", misck_checker->insecure ? "Yes" : "No");
+	dump_checker_opts(checker);
+}
+
+static bool
+misc_check_compare(void *a, void *b)
+{
+	misc_checker_t *old = CHECKER_DATA(a);
+	misc_checker_t *new = CHECKER_DATA(b);
+
+	if (strcmp(old->path, new->path) != 0)
+		return false;
+
+	return true;
 }
 
 static void
 misc_check_handler(__attribute__((unused)) vector_t *strvec)
 {
-	misc_checker_t *misck_checker = (misc_checker_t *) MALLOC(sizeof (misc_checker_t));
+	checker_t *checker;
 
-	misck_checker->uid = default_script_uid;
-	misck_checker->gid = default_script_gid;
+	new_misck_checker = (misc_checker_t *) MALLOC(sizeof (misc_checker_t));
+
+	script_user_set = false;
 
 	/* queue new checker */
-	queue_checker(free_misc_check, dump_misc_check, misc_check_thread,
-		      misck_checker, NULL);
+	checker = queue_checker(free_misc_check, dump_misc_check, misc_check_thread, misc_check_compare, new_misck_checker, NULL);
+
+	/* Set non-standard default value */
+	checker->default_retry = 0;
 }
 
 static void
 misc_path_handler(vector_t *strvec)
 {
-	misc_checker_t *misck_checker = CHECKER_GET();
-	misck_checker->path = CHECKER_VALUE_STRING(strvec);
+	if (!new_misck_checker)
+		return;
+
+	new_misck_checker->path = CHECKER_VALUE_STRING(strvec);
 }
 
 static void
 misc_timeout_handler(vector_t *strvec)
 {
-	misc_checker_t *misck_checker = CHECKER_GET();
-	misck_checker->timeout = CHECKER_VALUE_UINT(strvec) * TIMER_HZ;
+	if (!new_misck_checker)
+		return;
+
+	new_misck_checker->timeout = CHECKER_VALUE_UINT(strvec) * TIMER_HZ;
 }
 
 static void
 misc_dynamic_handler(__attribute__((unused)) vector_t *strvec)
 {
-	misc_checker_t *misck_checker = CHECKER_GET();
-	misck_checker->dynamic = 1;
+	if (!new_misck_checker)
+		return;
+
+	new_misck_checker->dynamic = true;
+
+	if (have_dynamic_misc_checker)
+		log_message(LOG_INFO, "Warning - more than one dynamic misc checker per real srver will cause problems");
+	else
+		have_dynamic_misc_checker = true;
 }
 
 static void
 misc_user_handler(vector_t *strvec)
 {
-	misc_checker_t *misck_checker = CHECKER_GET();
+	if (!new_misck_checker)
+		return;
 
 	if (vector_size(strvec) < 2) {
-		log_message(LOG_INFO, "No user specified for misc checker script %s", misck_checker->path);
+		log_message(LOG_INFO, "No user specified for misc checker script %s", new_misck_checker->path);
 		return;
 	}
 
-	if (set_script_uid_gid(strvec, 1, &misck_checker->uid, &misck_checker->gid))
-		log_message(LOG_INFO, "Failed to set uid/gid for misc checker script %s", misck_checker->path);
+	if (set_script_uid_gid(strvec, 1, &new_misck_checker->uid, &new_misck_checker->gid)) {
+		log_message(LOG_INFO, "Failed to set uid/gid for misc checker script %s - removing", new_misck_checker->path);
+		dequeue_new_checker();
+		new_misck_checker = NULL;
+	}
+	else
+		script_user_set = true;
+}
+
+static void
+misc_end_handler(void)
+{
+	if (!new_misck_checker)
+		return;
+
+	if (!new_misck_checker->path) {
+		log_message(LOG_INFO, "No script path has been specified for MISC_CHECKER - skipping");
+		dequeue_new_checker();
+		new_misck_checker = NULL;
+		return;
+	}
+
+	if (!script_user_set)
+	{
+		if ( set_default_script_user(NULL, NULL, global_data->script_security)) {
+			log_message(LOG_INFO, "Unable to set default user for misc script %s - removing", new_misck_checker->path);
+			dequeue_new_checker();
+			new_misck_checker = NULL;
+			return;
+		}
+
+		new_misck_checker->uid = default_script_uid;
+		new_misck_checker->gid = default_script_gid;
+	}
+
+	new_misck_checker = NULL;
 }
 
 void
@@ -120,11 +194,12 @@ install_misc_check_keyword(void)
 {
 	install_keyword("MISC_CHECK", &misc_check_handler);
 	install_sublevel();
+	install_checker_common_keywords(false);
 	install_keyword("misc_path", &misc_path_handler);
 	install_keyword("misc_timeout", &misc_timeout_handler);
 	install_keyword("misc_dynamic", &misc_dynamic_handler);
-	install_keyword("warmup", &warmup_handler);
 	install_keyword("user", &misc_user_handler);
+	install_sublevel_end_handler(&misc_end_handler);
 	install_sublevel_end();
 }
 
@@ -155,13 +230,16 @@ check_misc_script_security(void)
 
 		script_flags |= (flags = check_script_secure(&script, global_data->script_security, false));
 
+		/* The script path may have been updated if it wasn't an absolute path */
+		misc_script->path = script.name;
+
 		/* Mark not to run if needs inhibiting */
 		if (flags & SC_INHIBIT) {
 			log_message(LOG_INFO, "Disabling misc script %s due to insecure", misc_script->path);
 			misc_script->insecure = true;
 		}
 		else if (flags & SC_NOTFOUND) {
-			log_message(LOG_INFO, "Disabling misc script %s since not found", misc_script->path);
+			log_message(LOG_INFO, "Disabling misc script %s since not found/accessible", misc_script->path);
 			misc_script->insecure = true;
 		}
 		else if (!(flags & SC_EXECUTABLE))
@@ -189,22 +267,19 @@ misc_check_thread(thread_t * thread)
 	 * Register a new checker thread & return
 	 * if checker is disabled
 	 */
-	if (!CHECKER_ENABLED(checker)) {
+	if (!checker->enabled) {
 		/* Register next timer checker */
 		thread_add_timer(thread->master, misc_check_thread, checker,
-				 checker->vs->delay_loop);
+				 checker->delay_loop);
 		return 0;
 	}
 
 	misck_checker->forcing_termination = false;
 
-	/* Register next timer checker */
-	thread_add_timer(thread->master, misc_check_thread, checker,
-			 checker->vs->delay_loop);
-
 	/* Execute the script in a child process. Parent returns, child doesn't */
+	misck_checker->last_ran = time_now;
 	return system_call_script(thread->master, misc_check_child_thread,
-				  checker, (misck_checker->timeout) ? misck_checker->timeout : checker->vs->delay_loop,
+				  checker, (misck_checker->timeout) ? misck_checker->timeout : checker->delay_loop,
 				  misck_checker->path, misck_checker->uid, misck_checker->gid);
 }
 
@@ -215,6 +290,7 @@ misc_check_child_thread(thread_t * thread)
 	pid_t pid;
 	checker_t *checker;
 	misc_checker_t *misck_checker;
+	timeval_t next_time;
 
 	checker = THREAD_ARG(thread);
 	misck_checker = CHECKER_ARG(checker);
@@ -223,22 +299,25 @@ misc_check_child_thread(thread_t * thread)
 		pid = THREAD_CHILD_PID(thread);
 
 		/* The child hasn't responded. Kill it off. */
-		if (svr_checker_up(checker->id, checker->rs)) {
-			log_message(LOG_INFO, "Misc check to [%s] for [%s] timed out"
-					    , inet_sockaddrtos(&checker->rs->addr)
-					    , misck_checker->path);
-			smtp_alert(checker->rs, NULL, NULL,
-				   "DOWN",
-				   "=> MISC CHECK script timeout on service <=");
-			update_svr_checker_state(DOWN, checker->id
-						     , checker->vs
-						     , checker->rs);
+		if (checker->is_up) {
+			if (checker->retry_it < checker->retry)
+				checker->retry_it++;
+			else {
+				log_message(LOG_INFO, "Misc check to [%s] for [%s] timed out"
+						    , inet_sockaddrtos(&checker->rs->addr)
+						    , misck_checker->path);
+				smtp_alert(checker, NULL, NULL,
+					   "DOWN",
+					   "=> MISC CHECK script timeout on service <=");
+				update_svr_checker_state(DOWN, checker);
+				checker->retry_it = 0;
+			}
 		}
 
 		misck_checker->forcing_termination = true;
 		kill(-pid, SIGTERM);
 		thread_add_child(thread->master, misc_check_child_timeout_thread,
-				 checker, pid, 2);
+				 checker, pid, 2 * TIMER_HZ);
 		return 0;
 	}
 
@@ -248,39 +327,40 @@ misc_check_child_thread(thread_t * thread)
 		int status;
 		status = WEXITSTATUS(wait_status);
 		if (status == 0 ||
-		    (misck_checker->dynamic == 1 && status >= 2 && status <= 255)) {
+		    (misck_checker->dynamic && status >= 2 && status <= 255)) {
 			/*
 			 * The actual weight set when using misc_dynamic is two less than
 			 * the exit status returned.  Effective range is 0..253.
 			 * Catch legacy case of status being 0 but misc_dynamic being set.
 			 */
-			if (misck_checker->dynamic == 1 && status != 0)
+			if (misck_checker->dynamic && status != 0)
 				update_svr_wgt(status - 2, checker->vs,
 					       checker->rs, true);
 
 			/* everything is good */
-			if (!svr_checker_up(checker->id, checker->rs)) {
+			if (!checker->is_up) {
 				log_message(LOG_INFO, "Misc check to [%s] for [%s] success."
 						    , inet_sockaddrtos(&checker->rs->addr)
 						    , misck_checker->path);
-				smtp_alert(checker->rs, NULL, NULL,
+				smtp_alert(checker, NULL, NULL,
 					   "UP",
 					   "=> MISC CHECK succeed on service <=");
-				update_svr_checker_state(UP, checker->id
-							   , checker->vs
-							   , checker->rs);
+				update_svr_checker_state(UP, checker);
 			}
-		} else {
-			if (svr_checker_up(checker->id, checker->rs)) {
+
+			checker->retry_it = 0;
+		} else if (checker->is_up) {
+			if (checker->retry_it < checker->retry)
+				checker->retry_it++;
+			else {
 				log_message(LOG_INFO, "Misc check to [%s] for [%s] failed."
 						    , inet_sockaddrtos(&checker->rs->addr)
 						    , misck_checker->path);
-				smtp_alert(checker->rs, NULL, NULL,
+				smtp_alert(checker, NULL, NULL,
 					   "DOWN",
 					   "=> MISC CHECK failed on service <=");
-				update_svr_checker_state(DOWN, checker->id
-							     , checker->vs
-							     , checker->rs);
+				update_svr_checker_state(DOWN, checker);
+				checker->retry_it = 0;
 			}
 		}
 	}
@@ -296,6 +376,14 @@ misc_check_child_thread(thread_t * thread)
 
 	misck_checker->forcing_termination = false;
 
+	/* Register next timer checker */
+	next_time = timer_add_long(misck_checker->last_ran, checker->retry_it ? checker->delay_before_retry : checker->delay_loop);
+	next_time = timer_sub_now(next_time);
+	if (next_time.tv_sec < 0)
+		next_time.tv_sec = 0, next_time.tv_usec = 1;
+
+	thread_add_timer(thread->master, misc_check_thread, checker, timer_tol(next_time));
+
 	return 0;
 }
 
@@ -303,24 +391,36 @@ static int
 misc_check_child_timeout_thread(thread_t * thread)
 {
 	pid_t pid;
+	checker_t *checker;
 	misc_checker_t *misck_checker;
+	timeval_t next_time;
 
-	if (thread->type != THREAD_CHILD_TIMEOUT)
-		return 0;
-
-	/* OK, it still hasn't exited. Now really kill it off. */
-	pid = THREAD_CHILD_PID(thread);
-	if (kill(-pid, SIGKILL) < 0) {
-		/* Its possible it finished while we're handing this */
-		if (errno != ESRCH) {
-			DBG("kill error: %s", strerror(errno));
+	if (thread->type == THREAD_CHILD_TIMEOUT) {
+		/* OK, it still hasn't exited. Now really kill it off. */
+		pid = THREAD_CHILD_PID(thread);
+		if (kill(-pid, SIGKILL) < 0) {
+			/* Its possible it finished while we're handing this */
+			if (errno != ESRCH) {
+				DBG("kill error: %s", strerror(errno));
+			}
+			return 0;
 		}
-		return 0;
+
+		log_message(LOG_WARNING, "Process [%d] didn't respond to SIGTERM", pid);
 	}
 
-	log_message(LOG_WARNING, "Process [%d] didn't respond to SIGTERM", pid);
+	checker = THREAD_ARG(thread);
+	misck_checker = CHECKER_ARG(checker);
 
 	misck_checker->forcing_termination = false;
 
+	/* Register next timer checker */
+	next_time = timer_add_long(misck_checker->last_ran, checker->retry_it ? checker->delay_before_retry : checker->delay_loop);
+	next_time = timer_sub_now(next_time);
+	if (next_time.tv_sec < 0)
+		next_time.tv_sec = 0, next_time.tv_usec = 1;
+
+	thread_add_timer(thread->master, misc_check_thread, checker, timer_tol(next_time));
+
 	return 0;
 }
