@@ -22,6 +22,8 @@
 
 #include "config.h"
 
+#include <stdint.h>
+
 #include "check_data.h"
 #include "check_snmp.h"
 #include "list.h"
@@ -76,7 +78,6 @@ enum check_snmp_virtualserver_magic {
 	CHECK_SNMP_VSHYSTERESIS,
 	CHECK_SNMP_VSREALTOTAL,
 	CHECK_SNMP_VSREALUP,
-#ifdef _WITH_LVS_
 	CHECK_SNMP_VSSTATSCONNS,
 	CHECK_SNMP_VSSTATSINPKTS,
 	CHECK_SNMP_VSSTATSOUTPKTS,
@@ -106,7 +107,11 @@ enum check_snmp_virtualserver_magic {
 	CHECK_SNMP_VSSHFALLBACK,
 	CHECK_SNMP_VSSHPORT,
 	CHECK_SNMP_VSSCHED3,
-#endif
+	CHECK_SNMP_VSACTIONWHENDOWN,
+	CHECK_SNMP_VSRETRY,
+	CHECK_SNMP_VSDELAYBEFORERETRY,
+	CHECK_SNMP_VSWARMUP,
+	CHECK_SNMP_VSWEIGHT,
 };
 
 enum check_snmp_realserver_magic {
@@ -122,7 +127,6 @@ enum check_snmp_realserver_magic {
 	CHECK_SNMP_RSNOTIFYUP,
 	CHECK_SNMP_RSNOTIFYDOWN,
 	CHECK_SNMP_RSFAILEDCHECKS,
-#ifdef _WITH_LVS_
 	CHECK_SNMP_RSSTATSCONNS,
 	CHECK_SNMP_RSSTATSACTIVECONNS,
 	CHECK_SNMP_RSSTATSINACTIVECONNS,
@@ -149,22 +153,27 @@ enum check_snmp_realserver_magic {
 	CHECK_SNMP_RSRATEINBPSLOW,
 	CHECK_SNMP_RSRATEINBPSHIGH,
 	CHECK_SNMP_RSRATEOUTBPSLOW,
-	CHECK_SNMP_RSRATEOUTBPSHIGH
-#endif
+	CHECK_SNMP_RSRATEOUTBPSHIGH,
 #endif
+	CHECK_SNMP_RSLOADBALANCINGKIND,
+	CHECK_SNMP_RSVIRTUALHOST,
+	CHECK_SNMP_RSALPHA,
+	CHECK_SNMP_RSRETRY,
+	CHECK_SNMP_RSDELAYBEFORERETRY,
+	CHECK_SNMP_RSWARMUP,
+	CHECK_SNMP_RSDELAYLOOP
 };
 
 #define STATE_VSGM_FWMARK 1
-#define STATE_VSGM_ADDRESS 2
-#define STATE_VSGM_RANGE 3
-#define STATE_VSGM_END 4
+#define STATE_VSGM_ADDRESS_RANGE 2
+#define STATE_VSGM_END 3
 
 #define STATE_RS_SORRY 1
 #define STATE_RS_REGULAR_FIRST 2
 #define STATE_RS_REGULAR_NEXT 3
 #define STATE_RS_END 4
 
-#ifdef _WITH_LVS_
+#ifdef _WITH_VRRP_
 enum check_snmp_lvs_sync_daemon {
 	CHECK_SNMP_LVSSYNCDAEMONENABLED,
 	CHECK_SNMP_LVSSYNCDAEMONINTERFACE,
@@ -269,16 +278,13 @@ check_snmp_vsgroupmember(struct variable *vp, oid *name, size_t *length,
 		if (be)
 			break; /* Optimization: cannot be the lower anymore */
 		state = STATE_VSGM_FWMARK;
-		while (state != STATE_VSGM_END) {
+		while (state < STATE_VSGM_END) {
 			switch (state) {
 			case STATE_VSGM_FWMARK:
 				l = group->vfwmark;
 				break;
-			case STATE_VSGM_ADDRESS:
-				l = group->addr_ip;
-				break;
-			case STATE_VSGM_RANGE:
-				l = group->range;
+			case STATE_VSGM_ADDRESS_RANGE:
+				l = group->addr_range;
 				break;
 			default:
 				/* Dunno? */
@@ -355,13 +361,12 @@ check_snmp_vsgroupmember(struct variable *vp, oid *name, size_t *length,
 			struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *)&be->addr;
 			*var_len = 16;
 			memcpy(&ip6, &addr6->sin6_addr, sizeof(ip6));
-			ip6.s6_addr32[3] &= htonl(0xFFFFFF00);
-			ip6.s6_addr32[3] += htonl(be->range);
+			ip6.s6_addr16[7] = htons(ntohs(ip6.s6_addr16[7]) + be->range);
 			return (u_char *)&ip6;
 		} else {
 			struct sockaddr_in *addr4 = (struct sockaddr_in *)&be->addr;
 			*var_len = 4;
-			ip = (*(u_int32_t *)&addr4->sin_addr) & htonl(0xFFFFFF00);
+			ip = *(uint32_t *)&addr4->sin_addr;
 			ip += htonl(be->range);
 			return (u_char *)&ip;
 		}
@@ -425,10 +430,11 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 		long_ret.u = htons(inet_sockaddrport(&v->addr));
 		return (u_char *)&long_ret;
 	case CHECK_SNMP_VSPROTOCOL:
-		long_ret.u = (v->service_type == IPPROTO_TCP)?1:2;
+		long_ret.u = (v->service_type == IPPROTO_TCP) ? 1 :
+			     (v->service_type == IPPROTO_UDP) ? 2 :
+			     (v->service_type == IPPROTO_SCTP) ? 3 : 4;
 		return (u_char*)&long_ret;
 	case CHECK_SNMP_VSLOADBALANCINGALGO:
-#ifdef _WITH_LVS_
 		if (!strcmp(v->sched, "rr"))
 			long_ret.u = 1;
 		else if (!strcmp(v->sched, "wrr"))
@@ -449,14 +455,16 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 			long_ret.u = 9;
 		else if (!strcmp(v->sched, "nq"))
 			long_ret.u = 10;
+		else if (!strcmp(v->sched, "fo"))
+			long_ret.u = 11;
+		else if (!strcmp(v->sched, "ovf"))
+			long_ret.u = 12;
 		else
-#endif
 			long_ret.u = 99;
 		return (u_char*)&long_ret;
 	case CHECK_SNMP_VSLOADBALANCINGKIND:
 		long_ret.u = 0;
-#ifdef _WITH_LVS_
-		switch (v->loadbalancing_kind) {
+		switch (v->forwarding_method) {
 		case IP_VS_CONN_F_MASQ:
 			long_ret.u = 1;
 			break;
@@ -467,7 +475,6 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 			long_ret.u = 3;
 			break;
 		}
-#endif
 		if (!long_ret.u) break;
 		return (u_char*)&long_ret;
 	case CHECK_SNMP_VSSTATUS:
@@ -493,10 +500,7 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 		*var_len = sizeof(v->persistence_granularity);
 		return (u_char*)&v->persistence_granularity;
 	case CHECK_SNMP_VSDELAYLOOP:
-		if (v->delay_loop >= TIMER_MAX_SEC)
-			long_ret.u = v->delay_loop/TIMER_HZ;
-		else
-			long_ret.u = v->delay_loop;
+		long_ret.u = v->delay_loop/TIMER_HZ;
 		return (u_char*)&long_ret;
 	case CHECK_SNMP_VSHASUSPEND:
 		long_ret.u = v->ha_suspend?1:2;
@@ -516,16 +520,16 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 		long_ret.u = v->quorum;
 		return (u_char*)&long_ret;
 	case CHECK_SNMP_VSQUORUMSTATUS:
-		long_ret.u = v->quorum_state?1:2;
+		long_ret.u = v->quorum_state_up ? 1 : 2;
 		return (u_char*)&long_ret;
 	case CHECK_SNMP_VSQUORUMUP:
-		if (!v->quorum_up) break;
-		*var_len = strlen(v->quorum_up->name);
-		return (u_char*)v->quorum_up->name;
+		if (!v->notify_quorum_up) break;
+		*var_len = strlen(v->notify_quorum_up->name);
+		return (u_char*)v->notify_quorum_up->name;
 	case CHECK_SNMP_VSQUORUMDOWN:
-		if (!v->quorum_down) break;
-		*var_len = strlen(v->quorum_down->name);
-		return (u_char*)v->quorum_down->name;
+		if (!v->notify_quorum_down) break;
+		*var_len = strlen(v->notify_quorum_down->name);
+		return (u_char*)v->notify_quorum_down->name;
 	case CHECK_SNMP_VSHYSTERESIS:
 		long_ret.u = v->hysteresis;
 		return (u_char*)&long_ret;
@@ -542,7 +546,6 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 				if (((real_server_t *)ELEMENT_DATA(e))->alive)
 					long_ret.u++;
 		return (u_char*)&long_ret;
-#ifdef _WITH_LVS_
 	case CHECK_SNMP_VSSTATSCONNS:
 		ipvs_update_stats(v);
 		long_ret.u = v->stats.conns;
@@ -587,7 +590,6 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 		ipvs_update_stats(v);
 		long_ret.u = v->stats.outbps;
 		return (u_char*)&long_ret;
-#endif
 #ifdef _WITH_LVS_64BIT_STATS_
 	case CHECK_SNMP_VSSTATSCONNS64:
 		ipvs_update_stats(v);
@@ -662,6 +664,21 @@ check_snmp_virtualserver(struct variable *vp, oid *name, size_t *length,
 		long_ret.u = v->flags & IP_VS_SVC_F_SCHED3 ? 1 : 2;
 		return (u_char*)&long_ret;
 #endif
+	case CHECK_SNMP_VSACTIONWHENDOWN:
+		long_ret.u = v->inhibit?2:1;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_VSRETRY:
+		long_ret.u = v->retry;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_VSDELAYBEFORERETRY:
+		long_ret.u = v->delay_before_retry / TIMER_HZ;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_VSWARMUP:
+		long_ret.u = v->warmup / TIMER_HZ;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_VSWEIGHT:
+		long_ret.s = v->weight;
+		return (u_char*)&long_ret;
 	default:
 		return NULL;
 	}
@@ -698,7 +715,6 @@ check_snmp_realserver_weight(int action,
 		for (e1 = LIST_HEAD(check_data->vs); e1; ELEMENT_NEXT(e1)) {
 			vs = ELEMENT_DATA(e1);
 			if (--ivs == 0) {
-				if (LIST_ISEMPTY(vs->rs)) return SNMP_ERR_NOSUCHNAME;
 				if (vs->s_svr) {
 					/* We don't want to set weight
 					   of sorry server */
@@ -718,7 +734,7 @@ check_snmp_realserver_weight(int action,
 		if (action == RESERVE2)
 			break;
 		/* Commit: change values. There is no way to fail. */
-		update_svr_wgt((long)(*var_val), vs, rs, true);
+		update_svr_wgt((unsigned)(*var_val), vs, rs, true);
 		break;
 	}
 	return SNMP_ERR_NOERROR;
@@ -772,11 +788,6 @@ check_snmp_realserver(struct variable *vp, oid *name, size_t *length,
 				type = state++;
 				break;
 			case STATE_RS_REGULAR_FIRST:
-				if (LIST_ISEMPTY(vs->rs)) {
-					e = NULL;
-					state = STATE_RS_END;
-					break;
-				}
 				e2 = LIST_HEAD(vs->rs);
 				e = ELEMENT_DATA(e2);
 				type = state++;
@@ -848,6 +859,21 @@ check_snmp_realserver(struct variable *vp, oid *name, size_t *length,
 	case CHECK_SNMP_RSPORT:
 		long_ret.u = htons(inet_sockaddrport(&be->addr));
 		return (u_char *)&long_ret;
+	case CHECK_SNMP_RSLOADBALANCINGKIND:
+		long_ret.u = 0;
+		switch (be->forwarding_method) {
+		case IP_VS_CONN_F_MASQ:
+			long_ret.u = 1;
+			break;
+		case IP_VS_CONN_F_DROUTE:
+			long_ret.u = 2;
+			break;
+		case IP_VS_CONN_F_TUNNEL:
+			long_ret.u = 3;
+			break;
+		}
+		if (!long_ret.u) break;
+		return (u_char*)&long_ret;
 	case CHECK_SNMP_RSSTATUS:
 		if (btype == STATE_RS_SORRY) break;
 		long_ret.u = be->alive?1:2;
@@ -881,14 +907,14 @@ check_snmp_realserver(struct variable *vp, oid *name, size_t *length,
 		if (!be->notify_down) break;
 		*var_len = strlen(be->notify_down->name);
 		return (u_char*)be->notify_down->name;
+	case CHECK_SNMP_RSVIRTUALHOST:
+		if (!be->virtualhost) break;
+		*var_len = strlen(be->virtualhost);
+		return (u_char*)be->virtualhost;
 	case CHECK_SNMP_RSFAILEDCHECKS:
 		if (btype == STATE_RS_SORRY) break;
-		if (LIST_ISEMPTY(be->failed_checkers))
-			long_ret.u = 0;
-		else
-			long_ret.u = LIST_SIZE(be->failed_checkers);
+		long_ret.u = be->num_failed_checkers;
 		return (u_char*)&long_ret;
-#ifdef _WITH_LVS_
 	case CHECK_SNMP_RSSTATSCONNS:
 		ipvs_update_stats(bvs);
 		long_ret.u = be->stats.conns;
@@ -1005,7 +1031,21 @@ check_snmp_realserver(struct variable *vp, oid *name, size_t *length,
 		long_ret.u = be->stats.outbps >> 32;
 		return (u_char*)&long_ret;
 #endif
-#endif
+	case CHECK_SNMP_RSALPHA:
+		long_ret.u = be->alpha?1:2;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_RSRETRY:
+		long_ret.u = be->retry;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_RSDELAYBEFORERETRY:
+		long_ret.u = be->delay_before_retry / TIMER_HZ;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_RSWARMUP:
+		long_ret.u = be->warmup / TIMER_HZ;
+		return (u_char*)&long_ret;
+	case CHECK_SNMP_RSDELAYLOOP:
+		long_ret.u = be->delay_loop / TIMER_HZ;
+		return (u_char*)&long_ret;
 	default:
 		return NULL;
 	}
@@ -1017,7 +1057,7 @@ check_snmp_realserver(struct variable *vp, oid *name, size_t *length,
 	return NULL;
 }
 
-#ifdef _WITH_LVS_
+#ifdef _WITH_VRRP_
 static u_char*
 check_snmp_lvs_sync_daemon(struct variable *vp, oid *name, size_t *length,
 				 int exact, size_t *var_len, WriteMethod **write_method)
@@ -1183,7 +1223,6 @@ static struct variable8 check_vars[] = {
 	 check_snmp_virtualserver, 3, {3, 1, 25}},
 	{CHECK_SNMP_VSHYSTERESIS, ASN_UNSIGNED, RONLY,
 	 check_snmp_virtualserver, 3, {3, 1, 26}},
-#ifdef _WITH_LVS_
 	{CHECK_SNMP_VSSTATSCONNS, ASN_GAUGE, RONLY,
 	 check_snmp_virtualserver, 3, {3, 1, 27}},
 	{CHECK_SNMP_VSSTATSINPKTS, ASN_COUNTER, RONLY,
@@ -1231,7 +1270,6 @@ static struct variable8 check_vars[] = {
 	 check_snmp_virtualserver, 3, {3, 1, 49}},
 	{CHECK_SNMP_VSRATEOUTBPSHIGH, ASN_UNSIGNED, RONLY,
 	 check_snmp_virtualserver, 3, {3, 1, 50}},
-#endif
 #endif
 	{CHECK_SNMP_VSPERSISTGRANULARITY6, ASN_UNSIGNED, RONLY,
 	 check_snmp_virtualserver, 3, {3, 1, 51}},
@@ -1243,6 +1281,16 @@ static struct variable8 check_vars[] = {
 	 check_snmp_virtualserver, 3, {3, 1, 54}},
 	{CHECK_SNMP_VSSCHED3, ASN_INTEGER, RONLY,
 	 check_snmp_virtualserver, 3, {3, 1, 55}},
+	{CHECK_SNMP_VSACTIONWHENDOWN, ASN_INTEGER, RONLY,
+	 check_snmp_virtualserver, 3, {3, 1, 56}},
+	{CHECK_SNMP_VSRETRY, ASN_INTEGER, RONLY,
+	 check_snmp_virtualserver, 3, {3, 1, 57}},
+	{CHECK_SNMP_VSDELAYBEFORERETRY, ASN_INTEGER, RONLY,
+	 check_snmp_virtualserver, 3, {3, 1, 58}},
+	{CHECK_SNMP_VSWARMUP, ASN_INTEGER, RONLY,
+	 check_snmp_virtualserver, 3, {3, 1, 59}},
+	{CHECK_SNMP_VSWEIGHT, ASN_INTEGER, RONLY,
+	 check_snmp_virtualserver, 3, {3, 1, 60}},
 
 	/* realServerTable */
 	{CHECK_SNMP_RSTYPE, ASN_INTEGER, RONLY,
@@ -1269,7 +1317,6 @@ static struct variable8 check_vars[] = {
 	 check_snmp_realserver, 3, {4, 1, 12}},
 	{CHECK_SNMP_RSFAILEDCHECKS, ASN_UNSIGNED, RONLY,
 	 check_snmp_realserver, 3, {4, 1, 13}},
-#ifdef _WITH_LVS_
 	{CHECK_SNMP_RSSTATSCONNS, ASN_GAUGE, RONLY,
 	 check_snmp_realserver, 3, {4, 1, 14}},
 	{CHECK_SNMP_RSSTATSACTIVECONNS, ASN_GAUGE, RONLY,
@@ -1324,8 +1371,21 @@ static struct variable8 check_vars[] = {
 	{CHECK_SNMP_RSRATEOUTBPSHIGH, ASN_UNSIGNED, RONLY,
 	 check_snmp_realserver, 3, {4, 1, 39}},
 #endif
-#endif
-#ifdef _WITH_LVS_
+	{CHECK_SNMP_RSLOADBALANCINGKIND, ASN_UNSIGNED, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 40}},
+	{CHECK_SNMP_RSVIRTUALHOST, ASN_OCTET_STR, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 41}},
+	{CHECK_SNMP_RSALPHA, ASN_INTEGER, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 42}},
+	{CHECK_SNMP_RSRETRY, ASN_INTEGER, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 43}},
+	{CHECK_SNMP_RSDELAYBEFORERETRY, ASN_INTEGER, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 44}},
+	{CHECK_SNMP_RSWARMUP, ASN_INTEGER, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 45}},
+	{CHECK_SNMP_RSDELAYLOOP, ASN_INTEGER, RONLY,
+	 check_snmp_realserver, 3, {4, 1, 46}},
+#ifdef _WITH_VRRP_
 	/* LVS sync daemon configuration */
 	{CHECK_SNMP_LVSSYNCDAEMONENABLED, ASN_INTEGER, RONLY,
 	 check_snmp_lvs_sync_daemon, 2, {6, 1}},
@@ -1440,15 +1500,11 @@ check_snmp_rs_trap(real_server_t *rs, virtual_server_t *vs)
 		notification_oid[notification_oid_len - 1] = 2;
 
 	/* Initialize data */
-	if (LIST_ISEMPTY(vs->rs))
-		realtotal = 0;
-	else
-		realtotal = LIST_SIZE(vs->rs);
+	realtotal = LIST_SIZE(vs->rs);
 	realup = 0;
-	if (!LIST_ISEMPTY(vs->rs))
-		for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e))
-			if (((real_server_t *)ELEMENT_DATA(e))->alive)
-				realup++;
+	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e))
+		if (((real_server_t *)ELEMENT_DATA(e))->alive)
+			realup++;
 
 	/* snmpTrapOID */
 	snmp_varlist_add_variable(&notification_vars,
@@ -1542,7 +1598,7 @@ check_snmp_rs_trap(real_server_t *rs, virtual_server_t *vs)
 				  (u_char *)&vsprotocol,
 				  sizeof(vsprotocol));
 	if (!rs) {
-		quorumstatus = vs->quorum_state?1:2;
+		quorumstatus = vs->quorum_state_up ? 1 : 2;
 		snmp_varlist_add_variable(&notification_vars,
 					  quorumstatus_oid, quorumstatus_oid_len,
 					  ASN_INTEGER,
