@@ -50,45 +50,32 @@ free_tcp_check(void *data)
 static void
 dump_tcp_check(void *data)
 {
-	tcp_check_t *tcp_check = CHECKER_DATA(data);
+	checker_t *checker = data;
 
 	log_message(LOG_INFO, "   Keepalive method = TCP_CHECK");
-	dump_conn_opts(CHECKER_CO(data));
-	if (tcp_check->n_retry) {
-		log_message(LOG_INFO, "     Retry count = %d"
-			    , tcp_check->n_retry);
-		log_message(LOG_INFO, "     Retry delay = %ld"
-			    , tcp_check->delay_before_retry / TIMER_HZ);
-	}
+	dump_checker_opts(checker);
+}
+
+static bool
+tcp_check_compare(void *a, void *b)
+{
+	return compare_conn_opts(CHECKER_CO(a), CHECKER_CO(b));
 }
 
 static void
 tcp_check_handler(__attribute__((unused)) vector_t *strvec)
 {
-	tcp_check_t *tcp_check;
-
-	tcp_check = MALLOC(sizeof (tcp_check_t));
-	tcp_check->n_retry = 1;
-	tcp_check->delay_before_retry = 1 * TIMER_HZ;
-
 	/* queue new checker */
-	queue_checker(free_tcp_check, dump_tcp_check, tcp_connect_thread
-		      ,tcp_check, CHECKER_NEW_CO());
+	queue_checker(free_tcp_check, dump_tcp_check, tcp_connect_thread,
+		      tcp_check_compare, NULL, CHECKER_NEW_CO());
 }
 
 static void
-tcp_retry_handler(vector_t *strvec)
+tcp_check_end_handler(void)
 {
-	tcp_check_t *tcp_check = CHECKER_GET();
-	tcp_check->n_retry = CHECKER_VALUE_UINT(strvec);
-}
-
-static void
-tcp_delay_before_retry_handler(vector_t *strvec)
-{
-	tcp_check_t *tcp_check = CHECKER_GET();
-	tcp_check->delay_before_retry =
-		CHECKER_VALUE_UINT(strvec) * TIMER_HZ;
+	if (!check_conn_opts(CHECKER_GET_CO())) {
+		dequeue_new_checker();
+	}
 }
 
 void
@@ -96,53 +83,45 @@ install_tcp_check_keyword(void)
 {
 	install_keyword("TCP_CHECK", &tcp_check_handler);
 	install_sublevel();
-	install_keyword("retry", &tcp_retry_handler);
-	install_keyword("delay_before_retry", &tcp_delay_before_retry_handler);
-	install_connect_keywords();
-	install_keyword("warmup", &warmup_handler);
+	install_checker_common_keywords(true);
+	install_sublevel_end_handler(tcp_check_end_handler);
 	install_sublevel_end();
 }
 
 static void
-tcp_epilog(thread_t * thread, int is_success)
+tcp_epilog(thread_t * thread, bool is_success)
 {
 	checker_t *checker;
-	tcp_check_t *tcp_check;
 	unsigned long delay;
 
 	checker = THREAD_ARG(thread);
-	tcp_check = CHECKER_ARG(checker);
 
-	if (is_success || tcp_check->retry_it > tcp_check->n_retry - 1) {
-		delay = checker->vs->delay_loop;
-		tcp_check->retry_it = 0;
+	if (is_success || checker->retry_it >= checker->retry) {
+		delay = checker->delay_loop;
+		checker->retry_it = 0;
 
-		if (is_success && !svr_checker_up(checker->id, checker->rs)) {
+		if (is_success && !checker->is_up) {
 			log_message(LOG_INFO, "TCP connection to %s success."
 					, FMT_TCP_RS(checker));
-			smtp_alert(checker->rs, NULL, NULL,
+			smtp_alert(checker, NULL, NULL,
 				   "UP",
 				   "=> TCP CHECK succeed on service <=");
-			update_svr_checker_state(UP, checker->id
-						   , checker->vs
-						   , checker->rs);
-		} else if (! is_success
-			   && svr_checker_up(checker->id, checker->rs)) {
-			if (tcp_check->n_retry)
+			update_svr_checker_state(UP, checker);
+		} else if (!is_success
+			   && checker->is_up) {
+			if (checker->retry)
 				log_message(LOG_INFO
 				    , "Check on service %s failed after %d retry."
 				    , FMT_TCP_RS(checker)
-				    , tcp_check->n_retry);
-			smtp_alert(checker->rs, NULL, NULL,
+				    , checker->retry);
+			smtp_alert(checker, NULL, NULL,
 				   "DOWN",
 				   "=> TCP CHECK failed on service <=");
-			update_svr_checker_state(DOWN, checker->id
-						     , checker->vs
-						     , checker->rs);
+			update_svr_checker_state(DOWN, checker);
 		}
 	} else {
-		delay = tcp_check->delay_before_retry;
-		++tcp_check->retry_it;
+		delay = checker->delay_before_retry;
+		++checker->retry_it;
 	}
 
 	/* Register next timer checker */
@@ -167,19 +146,19 @@ tcp_check_thread(thread_t * thread)
 		break;
 	case connect_success:
 		close(thread->u.fd);
-		tcp_epilog(thread, 1);
+		tcp_epilog(thread, true);
 		break;
 	case connect_timeout:
-		if (svr_checker_up(checker->id, checker->rs))
+		if (checker->is_up)
 			log_message(LOG_INFO, "TCP connection to %s timeout."
 					, FMT_TCP_RS(checker));
-		tcp_epilog(thread, 0);
+		tcp_epilog(thread, false);
 		break;
 	default:
-		if (svr_checker_up(checker->id, checker->rs))
+		if (checker->is_up)
 			log_message(LOG_INFO, "TCP connection to %s failed."
 					, FMT_TCP_RS(checker));
-		tcp_epilog(thread, 0);
+		tcp_epilog(thread, false);
 	}
 
 	return 0;
@@ -197,16 +176,16 @@ tcp_connect_thread(thread_t * thread)
 	 * Register a new checker thread & return
 	 * if checker is disabled
 	 */
-	if (!CHECKER_ENABLED(checker)) {
+	if (!checker->enabled) {
 		thread_add_timer(thread->master, tcp_connect_thread, checker,
-				 checker->vs->delay_loop);
+				 checker->delay_loop);
 		return 0;
 	}
 
 	if ((fd = socket(co->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP)) == -1) {
 		log_message(LOG_INFO, "TCP connect fail to create socket. Rescheduling.");
 		thread_add_timer(thread->master, tcp_connect_thread, checker,
-				checker->vs->delay_loop);
+				checker->delay_loop);
 
 		return 0;
 	}
@@ -224,7 +203,7 @@ tcp_connect_thread(thread_t * thread)
 		close(fd);
 		log_message(LOG_INFO, "TCP socket bind failed. Rescheduling.");
 		thread_add_timer(thread->master, tcp_connect_thread, checker,
-				checker->vs->delay_loop);
+				checker->delay_loop);
 	}
 
 	return 0;
