@@ -24,6 +24,10 @@
 
 #include "git-commit.h"
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include <stdlib.h>
 #include <sys/utsname.h>
 #include <sys/resource.h>
@@ -31,23 +35,31 @@
 
 #include "main.h"
 #include "config.h"
+#include "git-commit.h"
 #include "signals.h"
 #include "pidfile.h"
 #include "bitops.h"
 #include "logger.h"
 #include "parser.h"
 #include "notify.h"
+#ifdef _WITH_LVS_
 #include "check_parser.h"
+#endif
+#ifdef _WITH_VRRP_
 #include "vrrp_parser.h"
+#ifdef _WITH_JSON_
+#include "vrrp_json.h"
+#endif
+#endif
 #include "global_parser.h"
 #if HAVE_DECL_CLONE_NEWNET
 #include "namespaces.h"
 #endif
-#include "vrrp_netlink.h"
+#include "keepalived_netlink.h"
 
 #define	LOG_FACILITY_MAX	7
-#define	VERSION_STRING		PACKAGE_NAME " v" PACKAGE_VERSION " (" VERSION_DATE ")"
-#define COPYRIGHT_STRING	"Copyright(C) 2001-" COPYRIGHT_YEAR " Alexandre Cassen, <acassen@gmail.com>"
+#define	VERSION_STRING		PACKAGE_NAME " v" PACKAGE_VERSION " (" GIT_DATE ")"
+#define COPYRIGHT_STRING	"Copyright(C) 2001-" GIT_YEAR " Alexandre Cassen, <acassen@gmail.com>"
 #define BUILD_OPTIONS		CONFIGURATION_OPTIONS
 
 #define CHILD_WAIT_SECS	5
@@ -75,13 +87,12 @@ const char *snmp_socket;				/* Socket to use for SNMP agent */
 #endif
 static char *syslog_ident;				/* syslog ident if not default */
 char *instance_name;					/* keepalived instance name */
-bool use_pid_dir;					/* Put pid files in /var/run/keepalived */
-size_t getpwnam_buf_len;				/* Buffer length needed for getpwnam_r/getgrname_r */
-uid_t default_script_uid;				/* Default user/group for script execution */
-gid_t default_script_gid;
+bool use_pid_dir;					/* Put pid files in /var/run/keepalived or @localstatedir@/run/keepalived */
+
 unsigned os_major;					/* Kernel version */
 unsigned os_minor;
 unsigned os_release;
+char *hostname;						/* Initial part of hostname */
 
 #if HAVE_DECL_CLONE_NEWNET
 char *network_namespace;				/* The network namespace we are running in */
@@ -115,6 +126,9 @@ free_parent_mallocs_startup(bool am_child)
 #else
 		free(syslog_ident);
 #endif
+
+		if (orig_core_dump_pattern)
+			FREE_PTR(orig_core_dump_pattern);
 	}
 
 	if (free_main_pidfile) {
@@ -435,10 +449,13 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 static void
 signal_init(void)
 {
-	signal_handler_init(1);
+	signal_handler_init();
 	signal_set(SIGHUP, propogate_signal, NULL);
 	signal_set(SIGUSR1, propogate_signal, NULL);
 	signal_set(SIGUSR2, propogate_signal, NULL);
+#ifdef _WITH_JSON_
+	signal_set(SIGJSON, propogate_signal, NULL);
+#endif
 	signal_set(SIGINT, sigend, NULL);
 	signal_set(SIGTERM, sigend, NULL);
 	signal_ignore(SIGPIPE);
@@ -457,10 +474,10 @@ update_core_dump_pattern(const char *pattern_str)
 	int fd;
 	bool initialising = (orig_core_dump_pattern == NULL);
 
-	/* CORENAME_MAX_SIZE in kernel source defines the maximum string length,
-	 * see core_pattern[CORENAME_MAX_SIZE] in fs/coredump.c. Currently,
-	 * (Linux 4.6) defineds it to be 128, but the definition is not exposed
-	 * to user-space. */
+	/* CORENAME_MAX_SIZE in kernel source include/linux/binfmts.h defines
+	 * the maximum string length, * see core_pattern[CORENAME_MAX_SIZE] in
+	 * fs/coredump.c. Currently (Linux 4.10) defines it to be 128, but the
+	 * definition is not exposed to user-space. */
 #define	CORENAME_MAX_SIZE	128
 
 	if (initialising)
@@ -510,7 +527,7 @@ core_dump_init(void)
 			log_message(LOG_INFO, "Failed to set core file size");
 	}
 }
-		
+
 /* Usage function */
 static void
 usage(const char *prog)
@@ -524,9 +541,13 @@ usage(const char *prog)
 	fprintf(stderr, "  -l, --log-console            Log messages to local console\n");
 	fprintf(stderr, "  -D, --log-detail             Detailed log messages\n");
 	fprintf(stderr, "  -S, --log-facility=[0-7]     Set syslog facility to LOG_LOCAL[0-7]\n");
+#ifdef _WITH_VRRP_
 	fprintf(stderr, "  -X, --release-vips           Drop VIP on transition from signal.\n");
 	fprintf(stderr, "  -V, --dont-release-vrrp      Don't remove VRRP VIPs and VROUTEs on daemon stop\n");
+#endif
+#ifdef _WITH_LVS_
 	fprintf(stderr, "  -I, --dont-release-ipvs      Don't remove IPVS topology on daemon stop\n");
+#endif
 	fprintf(stderr, "  -R, --dont-respawn           Don't respawn child processes\n");
 	fprintf(stderr, "  -n, --dont-fork              Don't fork the daemon process\n");
 	fprintf(stderr, "  -d, --dump-conf              Dump the configuration data\n");
@@ -536,6 +557,7 @@ usage(const char *prog)
 #endif
 #ifdef _WITH_LVS_
 	fprintf(stderr, "  -c, --checkers_pid=FILE      Use specified pidfile for checkers child process\n");
+	fprintf(stderr, "  -a, --address-monitoring     Report all address additions/deletions notified via netlink\n");
 #endif
 #ifdef _WITH_SNMP_
 	fprintf(stderr, "  -x, --snmp                   Enable SNMP subsystem\n");
@@ -549,66 +571,82 @@ usage(const char *prog)
 #ifdef _MEM_CHECK_LOG_
 	fprintf(stderr, "  -L, --mem-check-log          Log malloc/frees to syslog\n");
 #endif
-	fprintf(stderr, "  -i, --config_id id           Skip any configuration lines beginning '@' that don't match id\n");
+	fprintf(stderr, "  -i, --config-id id           Skip any configuration lines beginning '@' that don't match id\n"
+		        "                                or any lines beginning @^ that do match.\n"
+		        "                                The config-id defaults to the node name if option not used\n");
+	fprintf(stderr, "      --signum=SIGFUNC         Return signal number for STOP, RELOAD, DATA, STATS"
+#ifdef _WITH_JSON_
+								", JSON"
+#endif
+								"\n");
 	fprintf(stderr, "  -v, --version                Display the version number\n");
 	fprintf(stderr, "  -h, --help                   Display this help message\n");
 }
 
 /* Command line parser */
-static void
+static bool
 parse_cmdline(int argc, char **argv)
 {
 	int c;
+	bool reopen_log = false;
+	int signum;
 
 	struct option long_options[] = {
-		{"use-file",          required_argument, 0, 'f'},
+		{"use-file",		required_argument,	NULL, 'f'},
 #if defined _WITH_VRRP_ && defined _WITH_LVS_
-		{"vrrp",              no_argument,       0, 'P'},
-		{"check",             no_argument,       0, 'C'},
+		{"vrrp",		no_argument,		NULL, 'P'},
+		{"check",		no_argument,		NULL, 'C'},
 #endif
-		{"log-console",       no_argument,       0, 'l'},
-		{"log-detail",        no_argument,       0, 'D'},
-		{"log-facility",      required_argument, 0, 'S'},
-		{"release-vips",      no_argument,       0, 'X'},
-		{"dont-release-vrrp", no_argument,       0, 'V'},
-		{"dont-release-ipvs", no_argument,       0, 'I'},
-		{"dont-respawn",      no_argument,       0, 'R'},
-		{"dont-fork",         no_argument,       0, 'n'},
-		{"dump-conf",         no_argument,       0, 'd'},
-		{"pid",               required_argument, 0, 'p'},
+		{"log-console",		no_argument,		NULL, 'l'},
+		{"log-detail",		no_argument,		NULL, 'D'},
+		{"log-facility",	required_argument,	NULL, 'S'},
 #ifdef _WITH_VRRP_
-		{"vrrp_pid",          required_argument, 0, 'r'},
+		{"release-vips",	no_argument,		NULL, 'X'},
+		{"dont-release-vrrp",	no_argument,		NULL, 'V'},
 #endif
 #ifdef _WITH_LVS_
-		{"checkers_pid",      required_argument, 0, 'c'},
+		{"dont-release-ipvs",	no_argument,		NULL, 'I'},
+#endif
+		{"dont-respawn",	no_argument,		NULL, 'R'},
+		{"dont-fork",		no_argument,		NULL, 'n'},
+		{"dump-conf",		no_argument,		NULL, 'd'},
+		{"pid",			required_argument,	NULL, 'p'},
+#ifdef _WITH_VRRP_
+		{"vrrp_pid",		required_argument,	NULL, 'r'},
+#endif
+#ifdef _WITH_LVS_
+		{"checkers_pid",	required_argument,	NULL, 'c'},
+		{"address-monitoring",	no_argument,		NULL, 'a'},
 #endif
 #ifdef _WITH_SNMP_
-		{"snmp",              no_argument,       0, 'x'},
-		{"snmp-agent-socket", required_argument, 0, 'A'},
+		{"snmp",		no_argument,		NULL, 'x'},
+		{"snmp-agent-socket",	required_argument,	NULL, 'A'},
 #endif
-		{"core-dump",         no_argument,       0, 'm'},
-		{"core-dump-pattern", optional_argument, 0, 'M'},
+		{"core-dump",		no_argument,		NULL, 'm'},
+		{"core-dump-pattern",	optional_argument,	NULL, 'M'},
 #ifdef _MEM_CHECK_LOG_
-		{"mem-check-log",     no_argument,       0, 'L'},
+		{"mem-check-log",	no_argument,		NULL, 'L'},
 #endif
 #if HAVE_DECL_CLONE_NEWNET
-		{"namespace",         required_argument, 0, 's'},
+		{"namespace",		required_argument,	NULL, 's'},
 #endif	
-		{"config-id",         required_argument, 0, 'i'},
-		{"version",           no_argument,       0, 'v'},
-		{"help",              no_argument,       0, 'h'},
-		{0, 0, 0, 0}
+		{"config-id",		required_argument,	NULL, 'i'},
+		{"signum",		required_argument,	NULL,  1 },
+		{"version",		no_argument,		NULL, 'v'},
+		{"help",		no_argument,		NULL, 'h'},
+
+		{NULL,			0,			NULL,  0 }
 	};
 
-	while ((c = getopt_long(argc, argv, "vhlndVIDRS:f:p:i:mM"
+	while ((c = getopt_long(argc, argv, "vhlndDRS:f:p:i:mM"
 #if defined _WITH_VRRP_ && defined _WITH_LVS_
 					    "PC"
 #endif
 #ifdef _WITH_VRRP_ 
-					    "r:"
+					    "r:VX"
 #endif
 #ifdef _WITH_LVS_
-					    "c:"
+					    "ac:I"
 #endif
 #ifdef _WITH_SNMP_
 					    "xA:"
@@ -636,6 +674,7 @@ parse_cmdline(int argc, char **argv)
 			break;
 		case 'l':
 			__set_bit(LOG_CONSOLE_BIT, &debug);
+			reopen_log = true;
 			break;
 		case 'n':
 			__set_bit(DONT_FORK_BIT, &debug);
@@ -643,23 +682,30 @@ parse_cmdline(int argc, char **argv)
 		case 'd':
 			__set_bit(DUMP_CONF_BIT, &debug);
 			break;
+#ifdef _WITH_VRRP_
 		case 'V':
 			__set_bit(DONT_RELEASE_VRRP_BIT, &debug);
 			break;
+#endif
+#ifdef _WITH_LVS_
 		case 'I':
 			__set_bit(DONT_RELEASE_IPVS_BIT, &debug);
 			break;
+#endif
 		case 'D':
 			__set_bit(LOG_DETAIL_BIT, &debug);
 			break;
 		case 'R':
 			__set_bit(DONT_RESPAWN_BIT, &debug);
 			break;
+#ifdef _WITH_VRRP_
 		case 'X':
 			__set_bit(RELEASE_VIPS_BIT, &debug);
 			break;
+#endif
 		case 'S':
 			log_facility = LOG_FACILITY[atoi(optarg)].facility;
+			reopen_log = true;
 			break;
 		case 'f':
 			conf_file = optarg;
@@ -681,6 +727,9 @@ parse_cmdline(int argc, char **argv)
 		case 'c':
 			checkers_pidfile = optarg;
 			break;
+		case 'a':
+			__set_bit(LOG_ADDRESS_CHANGES, &debug);
+			break;
 #endif
 #ifdef _WITH_VRRP_
 		case 'r':
@@ -699,6 +748,7 @@ parse_cmdline(int argc, char **argv)
 			set_core_dump_pattern = true;
 			if (optarg && optarg[0])
 				core_dump_pattern = optarg;
+			/* ... falls through ... */
 		case 'm':
 			create_core_dump = true;
 			break;
@@ -714,7 +764,19 @@ parse_cmdline(int argc, char **argv)
 			break;
 #endif
 		case 'i':
-			config_id = optarg;
+			FREE_PTR(config_id);
+			config_id = MALLOC(strlen(optarg) + 1);
+			strcpy(config_id, optarg);
+			break;
+		case 1:
+			signum = get_signum(optarg);
+			if (signum == -1) {
+				fprintf(stderr, "Unknown sigfunc %s\n", optarg);
+				exit(1);
+			}
+
+			printf("%d\n", signum);
+			exit(0);
 			break;
 		default:
 			exit(0);
@@ -728,6 +790,8 @@ parse_cmdline(int argc, char **argv)
 			printf("%s ", argv[optind++]);
 		printf("\n");
 	}
+
+	return reopen_log;
 }
 
 /* Entry point */
@@ -737,11 +801,15 @@ keepalived_main(int argc, char **argv)
 	bool report_stopped = true;
 	struct utsname uname_buf;
 	char *end;
-	size_t buf_len;
 
 	/* Init debugging level */
 	debug = 0;
 
+	/* We are the parent process */
+#ifndef _DEBUG_
+	prog_type = PROG_TYPE_PARENT;
+#endif
+
 	/* Initialise pointer to child finding function */
 	set_child_finder(find_keepalived_child);
 
@@ -753,39 +821,13 @@ keepalived_main(int argc, char **argv)
 	__set_bit(DAEMON_CHECKERS, &daemon_mode);
 #endif
 
-	/*
-	 * Parse command line and set debug level.
-	 * bits 0..7 reserved by main.c
-	 */
-	parse_cmdline(argc, argv);
-
-	openlog(PACKAGE_NAME, LOG_PID | ((__test_bit(LOG_CONSOLE_BIT, &debug)) ? LOG_CONS : 0) , log_facility);
-
-	if (__test_bit(LOG_CONSOLE_BIT, &debug))
-		enable_console_log();
-
-#ifdef GIT_COMMIT
-	log_message(LOG_INFO, "Starting %s, git commit %s", version_string, GIT_COMMIT);
-#else
-	log_message(LOG_INFO, "Starting %s", version_string);
-#endif
+	/* Open log with default settings so we can log initially */
+	openlog(PACKAGE_NAME, LOG_PID, log_facility);
 
 #ifdef _MEM_CHECK_
 	mem_log_init(PACKAGE_NAME, "Parent process");
 #endif
 
-	/* Handle any core file requirements */
-	core_dump_init();
-
-	netlink_set_recv_buf_size();
-
-	set_default_script_user(&default_script_uid, &default_script_gid);
-
-	/* Get buffer length needed for getpwnam_r/getgrnam_r */
-	getpwnam_buf_len = (size_t)sysconf(_SC_GETPW_R_SIZE_MAX);
-	if ((buf_len = (size_t)sysconf(_SC_GETGR_R_SIZE_MAX)) > getpwnam_buf_len)
-		getpwnam_buf_len = buf_len;
-
 	/* Some functionality depends on kernel version, so get the version here */
 	if (uname(&uname_buf))
 		log_message(LOG_INFO, "Unable to get uname() information - error %d", errno);
@@ -805,8 +847,39 @@ keepalived_main(int argc, char **argv)
 		}
 		if (!os_major)
 			log_message(LOG_INFO, "Unable to parse kernel version %s", uname_buf.release);
+
+		/* config_id defaults to hostname */
+		if (!config_id) {
+			end = strchrnul(uname_buf.nodename, '.');
+			config_id = MALLOC(end - uname_buf.nodename + 1);
+			strncpy(config_id, uname_buf.nodename, end - uname_buf.nodename);
+			config_id[end - uname_buf.nodename] = '\0';
+		}
 	}
 
+	/*
+	 * Parse command line and set debug level.
+	 * bits 0..7 reserved by main.c
+	 */
+	if (parse_cmdline(argc, argv)) {
+		closelog();
+		openlog(PACKAGE_NAME, LOG_PID | ((__test_bit(LOG_CONSOLE_BIT, &debug)) ? LOG_CONS : 0) , log_facility);
+	}
+
+	if (__test_bit(LOG_CONSOLE_BIT, &debug))
+		enable_console_log();
+
+#ifdef GIT_COMMIT
+	log_message(LOG_INFO, "Starting %s, git commit %s", version_string, GIT_COMMIT);
+#else
+	log_message(LOG_INFO, "Starting %s", version_string);
+#endif
+
+	/* Handle any core file requirements */
+	core_dump_init();
+
+	netlink_set_recv_buf_size();
+
 	/* Check we can read the configuration file(s).
 	   NOTE: the working directory will be / if we
 	   forked, but will be the current working directory
@@ -970,6 +1043,8 @@ end:
 
 	closelog();
 
+	FREE(config_id);
+
 #ifndef _MEM_CHECK_LOG_
 	FREE_PTR(syslog_ident);
 #else
