@@ -23,34 +23,40 @@
 #ifndef _CHECK_API_H
 #define _CHECK_API_H
 
+/* global includes */
+#include <stdbool.h>
+
 /* local includes */
 #include "check_data.h"
 #include "scheduler.h"
-
-/* connection options structure definition */
-typedef struct _conn_opts {
-	struct sockaddr_storage		dst;
-	struct sockaddr_storage		bindto;
-	unsigned int			connection_to; /* connection time-out */
-#ifdef _WITH_SO_MARK_
-	unsigned int			fwmark; /* to mark packets going out of the socket using SO_MARK */
-#endif
-} conn_opts_t;
+#include "layer4.h"
 
 /* Checkers structure definition */
 typedef struct _checker {
 	void				(*free_func) (void *);
 	void				(*dump_func) (void *);
 	int				(*launch) (struct _thread *);
-	virtual_server_t		*vs;	/* pointer to the checker thread virtualserver */
-	real_server_t			*rs;	/* pointer to the checker thread realserver */
+	bool				(*compare) (void *, void *);
+	virtual_server_t		*vs;			/* pointer to the checker thread virtualserver */
+	real_server_t			*rs;			/* pointer to the checker thread realserver */
 	void				*data;
-	checker_id_t			id;	/* Checker identifier */
-	int				enabled;/* Activation flag */
-	conn_opts_t			*co; /* connection options */
-	unsigned long			warmup;	/* max random timeout to start checker */
+	bool				enabled;		/* Activation flag */
+	bool				is_up;			/* Set if checker is up */
+	conn_opts_t			*co;			/* connection options */
+	int				alpha;			/* Alpha mode enabled */
+	unsigned long			delay_loop;		/* Interval between running checker */
+	unsigned long			warmup;			/* max random timeout to start checker */
+	unsigned			retry;			/* number of retries before failing */
+	unsigned long			delay_before_retry;	/* interval between retries */
+	unsigned			retry_it;		/* number of successive failures */
+	unsigned			default_retry;		/* number of retries before failing */
+	unsigned long			default_delay_before_retry; /* interval between retries */
+
 } checker_t;
 
+/* Typedefs */
+typedef checker_t * checker_id_t;
+
 /* Checkers queue */
 extern list checkers_queue;
 
@@ -64,27 +70,29 @@ extern list checkers_queue;
 #define CHECKER_VALUE_INT(X) (atoi(vector_slot(X,1)))
 #define CHECKER_VALUE_UINT(X) ((unsigned)strtoul(vector_slot(X,1), NULL, 10))
 #define CHECKER_VALUE_STRING(X) (set_value(X))
-#define CHECKER_VHOST(C) (VHOST((C)->vs))
-#define CHECKER_ENABLED(C) ((C)->enabled)
-#define CHECKER_ENABLE(C)  ((C)->enabled = 1)
-#define CHECKER_DISABLE(C) ((C)->enabled = 0)
 #define CHECKER_HA_SUSPEND(C) ((C)->vs->ha_suspend)
 #define CHECKER_NEW_CO() ((conn_opts_t *) MALLOC(sizeof (conn_opts_t)))
-#define FMT_CHK(C) FMT_RS((C)->rs)
+#define FMT_CHK(C) FMT_RS((C)->rs, (C)->vs)
 
 /* Prototypes definition */
 extern void init_checkers_queue(void);
-extern void dump_conn_opts(void *);
-extern void queue_checker(void (*free_func) (void *), void (*dump_func) (void *)
+extern void free_vs_checkers(virtual_server_t *);
+extern void dump_connection_opts(void *);
+extern void dump_checker_opts(void *);
+extern checker_t *queue_checker(void (*free_func) (void *), void (*dump_func) (void *)
 			  , int (*launch) (thread_t *)
+			  , bool (*compare) (void *, void *)
 			  , void *
 			  , conn_opts_t *);
+extern void dequeue_new_checker(void);
+extern bool check_conn_opts(conn_opts_t *);
+extern bool compare_conn_opts(conn_opts_t *, conn_opts_t *);
 extern void dump_checkers_queue(void);
 extern void free_checkers_queue(void);
 extern void register_checkers_thread(void);
 extern void install_checkers_keyword(void);
-extern void install_connect_keywords(void);
-extern void warmup_handler(vector_t *);
-extern void update_checker_activity(sa_family_t, void *, int);
+extern void checker_set_dst_port(struct sockaddr_storage *, uint16_t);
+extern void install_checker_common_keywords(bool);
+extern void update_checker_activity(sa_family_t, void *, bool);
 
 #endif
