@@ -26,18 +26,27 @@
 
 /* system includes */
 #include <stdlib.h>
+#include <stdbool.h>
+#include <sys/types.h>
 
 /* local includes */
 #include "scheduler.h"
 
 /* Checker argument structure  */
 typedef struct _misc_checker {
-	char *path;
-	long timeout;
-	int dynamic;  /* 0: old-style, 1: exit code from checker affects weight */
+	char			*path;
+	unsigned long		timeout;
+	bool			dynamic;	/* false: old-style, true: exit code from checker affects weight */
+	bool			forcing_termination; /* Set if we have sent the process a SIGTERM */
+	uid_t			uid;		/* uid for script execution */
+	gid_t			gid;		/* gid for script execution */
+	bool			insecure;	/* script is insecure */
+	timeval_t		last_ran;	/* Time script last ran */
 } misc_checker_t;
 
 /* Prototypes defs */
+extern void clear_dynamic_misc_check_flag(void);
 extern void install_misc_check_keyword(void);
+extern int check_misc_script_security(void);
 
 #endif
