@@ -31,12 +31,6 @@
 /* local includes */
 #include "scheduler.h"
 
-typedef struct _tcp_check {
-	int n_retry;
-	long delay_before_retry;
-	int retry_it;
-} tcp_check_t;
-
 /* macro utility */
 #define FMT_TCP_RS(C) FMT_CHK(C)
 
