@@ -25,6 +25,7 @@
 
 /* system includes */
 #include <syslog.h>
+#include <stdbool.h>
 
 /* locale includes */
 #include "check_data.h"
@@ -38,8 +39,8 @@
 #define IP_FW_CMD_DEL 0x0002
 
 /* UP & DOWN value */
-#define UP   1
-#define DOWN 0
+#define UP   true
+#define DOWN false
 
 /* LVS command set by kernel */
 #define LVS_CMD_ADD		IP_VS_SO_SET_ADD
@@ -49,12 +50,13 @@
 #define LVS_CMD_EDIT_DEST	IP_VS_SO_SET_EDITDEST
 
 /* prototypes */
-extern void update_svr_wgt(int, virtual_server_t *, real_server_t *, int);
-extern int svr_checker_up(checker_id_t, real_server_t *);
-extern void update_svr_checker_state(int, checker_id_t, virtual_server_t *, real_server_t *);
-extern int init_services(void);
-extern int clear_services(void);
-extern int clear_diff_services(void);
+extern void update_svr_wgt(int, virtual_server_t *, real_server_t *, bool);
+extern void set_checker_state(checker_t *, bool);
+extern void update_svr_checker_state(bool, checker_t *);
+extern bool init_services(void);
+extern void clear_services(void);
+extern void set_quorum_states(void);
+extern void clear_diff_services(list);
 extern void link_vsg_to_vs(void);
 
 #endif
