@@ -33,11 +33,15 @@
 #include <paths.h>
 
 /* lock pidfile */
+#ifdef GNU_STD_PATHS
+#define PID_DIR			LOCAL_STATE_DIR "/run/"
+#else
 #define PID_DIR			_PATH_VARRUN
+#endif
 #define KEEPALIVED_PID_DIR	PID_DIR PACKAGE "/"
 #define KEEPALIVED_PID_FILE	PACKAGE
 #define VRRP_PID_FILE		"vrrp"
-#define CHECKERS_PID_FILE 	"checkers"
+#define CHECKERS_PID_FILE	"checkers"
 #define	PID_EXTENSION		".pid"
 
 extern const char *pid_directory;
