@@ -26,8 +26,13 @@
 #include "vrrp_if.h"
 
 /* prototypes */
+extern void set_promote_secondaries(interface_t*);
+extern void reset_promote_secondaries(interface_t*);
+#ifdef _HAVE_VRRP_VMAC_
+extern void restore_rp_filter(void);
 extern void set_interface_parameters(const interface_t*, interface_t*);
 extern void reset_interface_parameters(interface_t*);
+#endif
 extern void link_disable_ipv6(const interface_t*);
 extern int get_ipv6_forwarding(const interface_t*);
 
