@@ -24,15 +24,14 @@
 #define _VRRP_IPSET_H
 
 #define LIBIPSET_NFPROTO_H
-#include <libipset/session.h>
 #include "vrrp_ipaddress.h"
 
-int add_ipsets(bool);
-int remove_ipsets(void);
-bool has_ipset_setname(struct ipset_session*, const char *);
+bool add_ipsets(bool);
+bool remove_ipsets(void);
+bool has_ipset_setname(void*, const char *);
 bool ipset_init(void);
-struct ipset_session* ipset_session_start(void);
-void ipset_session_end(struct ipset_session*);
-void ipset_entry(struct ipset_session*, int cmd, const ip_address_t*);
+void* ipset_session_start(void);
+void ipset_session_end(void*);
+void ipset_entry(void*, int cmd, const ip_address_t*);
 
 #endif
