@@ -29,6 +29,8 @@
 #include <sys/types.h>
 #include <string.h>
 #include <stdint.h>
+#include <unistd.h>
+#include <stdbool.h>
 
 /* local includes */
 #include "scheduler.h"
@@ -62,10 +64,11 @@ do {						\
 } while (0)
 
 /* extern prototypes */
-extern void vrrp_dispatcher_release(vrrp_data_t *);
 extern int vrrp_dispatcher_init(thread_t *);
+extern void vrrp_dispatcher_release(vrrp_data_t *);
 extern int vrrp_lower_prio_gratuitous_arp_thread(thread_t *);
 extern void vrrp_set_effective_priority(vrrp_t *, uint8_t);
+extern bool vrrp_child_finder(pid_t, char const **);
 extern int vrrp_arp_thread(thread_t *);
 
 #endif
