@@ -23,13 +23,14 @@
 #include "config.h"
 
 #include <string.h>
+#include <sys/prctl.h>
 
 #include "vrrp_daemon.h"
 #include "vrrp_scheduler.h"
 #include "vrrp_if.h"
 #include "vrrp_arp.h"
 #include "vrrp_ndisc.h"
-#include "vrrp_netlink.h"
+#include "keepalived_netlink.h"
 #include "vrrp_ipaddress.h"
 #include "vrrp_iptables.h"
 #ifdef _HAVE_FIB_ROUTING_
@@ -65,14 +66,39 @@
 #include "main.h"
 #include "memory.h"
 #include "parser.h"
+#ifdef _LIBNL_DYNAMIC_
+#include "libnl_link.h"
+#endif
+#ifdef _WITH_JSON_
+#include "vrrp_json.h"
+#endif
 
 /* Forward declarations */
 static int print_vrrp_data(thread_t * thread);
 static int print_vrrp_stats(thread_t * thread);
+#ifdef _WITH_JSON_
+static int print_vrrp_json(thread_t * thread);
+#endif
 static int reload_vrrp_thread(thread_t * thread);
 
 static char *vrrp_syslog_ident;
 
+#ifdef _WITH_LVS_
+static bool
+vrrp_ipvs_needed(void)
+{
+	return !!(global_data->lvs_syncd.ifname);
+}
+#endif
+
+static int
+vrrp_notify_fifo_script_exit(__attribute__((unused)) thread_t *thread)
+{
+	log_message(LOG_INFO, "vrrp notify fifo script terminated");
+
+	return 0;
+}
+
 /* Daemon stop sequence */
 static void
 stop_vrrp(int status)
@@ -136,6 +162,9 @@ stop_vrrp(int status)
 		dbus_stop();
 #endif
 
+	if (global_data->vrrp_notify_fifo.fd != -1)
+		notify_fifo_close(&global_data->notify_fifo, &global_data->vrrp_notify_fifo);
+
 	free_global_data(global_data);
 	free_vrrp_data(vrrp_data);
 	free_vrrp_buffer();
@@ -150,6 +179,8 @@ stop_vrrp(int status)
 
 	closelog();
 
+	FREE(config_id);
+
 #ifndef _MEM_CHECK_LOG_
 	FREE_PTR(vrrp_syslog_ident);
 #else
@@ -178,6 +209,7 @@ start_vrrp(void)
 		stop_vrrp(KEEPALIVED_EXIT_FATAL);
 		return;
 	}
+
 	init_data(conf_file, vrrp_init_keywords);
 
 	init_global_data(global_data);
@@ -189,10 +221,6 @@ start_vrrp(void)
 	if (global_data->vrrp_no_swap)
 		set_process_dont_swap(4096);	/* guess a stack size to reserve */
 
-#ifdef _HAVE_LIBIPTC_
-	iptables_init();
-#endif
-
 #ifdef _WITH_SNMP_
 	if (!reload && (global_data->enable_snmp_keepalived || global_data->enable_snmp_rfcv2 || global_data->enable_snmp_rfcv3)) {
 		vrrp_snmp_agent_init(global_data->snmp_socket);
@@ -255,10 +283,25 @@ start_vrrp(void)
 		return;
 	}
 
+	/* We need to delay the init of iptables to after vrrp_complete_init()
+	 * has been called so we know whether we want IPv4 and/or IPv6 */
+	iptables_init();
+
+	/* Create a notify FIFO if needed, and open it */
+	if (global_data->vrrp_notify_fifo.name)
+		notify_fifo_open(&global_data->notify_fifo, &global_data->vrrp_notify_fifo, vrrp_notify_fifo_script_exit, "vrrp_");
+
+	/* Make sure we don't have any old iptables/ipsets settings left around */
 #ifdef _HAVE_LIBIPTC_
+	if (!reload)
+		iptables_cleanup();
+
 	iptables_startup(reload);
 #endif
 
+	if (!reload)
+		vrrp_restore_interfaces_startup();
+
 	/* clear_diff_vrrp must be called after vrrp_complete_init, since the latter
 	 * sets ifa_index on the addresses, which is used for the address comparison */
 	if (reload)
@@ -324,6 +367,16 @@ sigusr2_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 	thread_add_event(master, print_vrrp_stats, NULL, 0);
 }
 
+#ifdef _WITH_JSON_
+static void
+sigjson_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
+{
+	log_message(LOG_INFO, "Printing VRRP as json for process(%d) on signal",
+		getpid());
+	thread_add_event(master, print_vrrp_json, NULL, 0);
+}
+#endif
+
 /* Terminate handler */
 static void
 sigend_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
@@ -336,12 +389,15 @@ sigend_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 static void
 vrrp_signal_init(void)
 {
-	signal_handler_init(0);
+	signal_handler_child_clear();
 	signal_set(SIGHUP, sighup_vrrp, NULL);
 	signal_set(SIGINT, sigend_vrrp, NULL);
 	signal_set(SIGTERM, sigend_vrrp, NULL);
 	signal_set(SIGUSR1, sigusr1_vrrp, NULL);
 	signal_set(SIGUSR2, sigusr2_vrrp, NULL);
+#ifdef _WITH_JSON_
+	signal_set(SIGJSON, sigjson_vrrp, NULL);
+#endif
 	signal_ignore(SIGPIPE);
 }
 
@@ -367,6 +423,9 @@ reload_vrrp_thread(__attribute__((unused)) thread_t * thread)
 		       true, false);
 #endif
 
+	/* Remove the notify fifo - we don't know if it will be the same after a reload */
+	notify_fifo_close(&global_data->notify_fifo, &global_data->vrrp_notify_fifo);
+
 	free_global_data(global_data);
 	free_vrrp_buffer();
 	gratuitous_arp_close();
@@ -418,6 +477,15 @@ print_vrrp_stats(__attribute__((unused)) thread_t * thread)
 	return 0;
 }
 
+#ifdef _WITH_JSON_
+static int
+print_vrrp_json(__attribute__((unused)) thread_t * thread)
+{
+	vrrp_print_json();
+	return 0;
+}
+#endif
+
 /* VRRP Child respawning thread */
 #ifndef _DEBUG_
 static int
@@ -472,9 +540,12 @@ start_vrrp_child(void)
 				 pid, RESPAWN_TIMER);
 		return 0;
 	}
+	prctl(PR_SET_PDEATHSIG, SIGTERM);
 
 	signal_handler_destroy();
 
+	prog_type = PROG_TYPE_VRRP;
+
 	/* Opening local VRRP syslog channel */
 	if ((instance_name
 #if HAVE_DECL_CLONE_NEWNET
@@ -515,6 +586,10 @@ start_vrrp_child(void)
 	/* Signal handling initialization */
 	vrrp_signal_init();
 
+#ifdef _LIBNL_DYNAMIC_
+	libnl_init();
+#endif
+
 	/* Start VRRP daemon */
 	start_vrrp();
 
