@@ -33,20 +33,33 @@
 #include <unistd.h>
 #define LIBIPSET_NFPROTO_H
 #define LIBIPSET_NF_INET_ADDR_H
+#if defined LIBIPSET_H_ADD_UAPI_IP_SET_H_GUARD || defined LIBIPSET_H_ADD_IP_SET_H_GUARD
+#include <linux/netfilter/ipset/ip_set.h>
+#if defined LIBIPSET_H_ADD_UAPI_IP_SET_H_GUARD
+#define _UAPI_IP_SET_H
+#else
+#define _IP_SET_H
+#endif
+#endif
+#include <libipset/session.h>
 #include <libipset/types.h>
 #include <netinet/in.h>
-#include <linux/types.h>        /* For __beXX types in userland */
-#include <linux/netfilter.h>    /* For nf_inet_addr */
+#include <linux/types.h>	/* For __beXX types in userland */
+#include <linux/netfilter.h>	/* For nf_inet_addr */
 #include <stdint.h>
-#include <dlfcn.h>
+#include <stdio.h>
 
 #include "logger.h"
 #include "global_data.h"
 #include "vrrp_iptables.h"
 #include "vrrp_ipset.h"
 #include "vrrp_ipaddress.h"
+#include "vrrp.h"
 #include "main.h"
 
+#ifdef _LIBIPSET_DYNAMIC_
+#include <dlfcn.h>
+
 /* The addresses of the functions we want */
 struct ipset_session* (*ipset_session_init_addr)(ipset_outfn outfn);
 int (*ipset_session_fini_addr)(struct ipset_session *session);
@@ -71,6 +84,9 @@ void (*ipset_load_types_addr)(void);
 #define ipset_load_types (*ipset_load_types_addr)
 
 static void* libipset_handle;
+#else
+#define ipset_cmd1 ipset_cmd
+#endif
 
 static bool
 do_ipset_cmd(struct ipset_session* session, enum ipset_cmd cmd, const char *setname,
@@ -134,14 +150,16 @@ ipset_destroy(struct ipset_session* session, const char *setname)
 }
 
 bool
-has_ipset_setname(struct ipset_session* session, const char *setname)
+has_ipset_setname(void* vsession, const char *setname)
 {
+	struct ipset_session *session = vsession;
+ 
 	ipset_session_data_set(session, IPSET_SETNAME, setname);
 
 	return ipset_cmd1(session, IPSET_CMD_HEADER, 0) == 0;
 }
 
-static int create_sets(const char* addr4, const char* addr6, const char* addr_if6, bool reload)
+static bool create_sets(const char* addr4, const char* addr6, const char* addr_if6, bool reload)
 {
 	struct ipset_session *session;
 
@@ -156,12 +174,12 @@ static int create_sets(const char* addr4, const char* addr6, const char* addr_if
 	if (!reload)
 		ipset_envopt_parse(session, IPSET_ENV_EXIST, NULL);
 
-	if (use_ip4tables) {
+	if (block_ipv4) {
 		if (!reload || !has_ipset_setname(session, addr4))
 			ipset_create(session, addr4, "hash:ip", NFPROTO_IPV4);
 	}
 
-	if (use_ip6tables) {
+	if (block_ipv6) {
 		if (!reload || !has_ipset_setname(session, addr6))
 			ipset_create(session, addr6, "hash:ip", NFPROTO_IPV6);
 		if (!reload || !has_ipset_setname(session, addr_if6)) {
@@ -179,10 +197,36 @@ static int create_sets(const char* addr4, const char* addr6, const char* addr_if
 	return true;
 }
 
+static
+bool set_match_loaded(void)
+{
+	char buf[XT_FUNCTION_MAXNAMELEN+1];
+	FILE *fp;
+	bool found = false;
+
+	fp = fopen( "/proc/net/ip_tables_matches", "r");
+	if (!fp)
+		return false;
+
+	while (fgets(buf, sizeof(buf), fp)) {
+		if ((buf[3] == '\0' || buf[3] == '\n') &&
+		    !strncmp(buf, "set", 3)) {
+			found = true;
+			break;
+		}
+	}
+
+	fclose(fp);
+
+	return found;
+}
+
 bool ipset_init(void)
 {
+#ifdef _LIBIPSET_DYNAMIC_
 	if (libipset_handle)
 		return true;
+#endif
 
 #if HAVE_DECL_CLONE_NEWNET
 	/* Don't attempt to use ipsets if running in a namespace and the default
@@ -198,51 +242,60 @@ bool ipset_init(void)
 	}
 #endif
 
+#ifdef _LIBIPSET_DYNAMIC_
 	/* Attempt to open the ipset library */
 	if (!(libipset_handle = dlopen("libipset.so", RTLD_NOW)) &&
-	    !(libipset_handle = dlopen("libipset.so.3", RTLD_NOW)) &&
-	    !(libipset_handle = dlopen("libipset.so.2", RTLD_NOW))) {
-		/* Generate the most useful error message */
-		dlopen("libipset.so.3", RTLD_NOW);
-
+	    !(libipset_handle = dlopen(IPSET_LIB_NAME, RTLD_NOW))) {
 		log_message(LOG_INFO, "Unable to load ipset library - %s", dlerror());
 		return false;
 	}
 
-	ipset_session_init_addr = dlsym(libipset_handle, "ipset_session_init");
-	ipset_session_fini_addr = dlsym(libipset_handle, "ipset_session_fini");
-	ipset_session_data_addr = dlsym(libipset_handle,"ipset_session_data");
-	ipset_session_error_addr = dlsym(libipset_handle,"ipset_session_error");
-	ipset_envopt_parse_addr = dlsym(libipset_handle,"ipset_envopt_parse");
-	ipset_type_get_addr = dlsym(libipset_handle,"ipset_type_get");
-	ipset_data_set_addr = dlsym(libipset_handle,"ipset_data_set");
-	ipset_cmd_addr = dlsym(libipset_handle,"ipset_cmd");
-	ipset_load_types_addr = dlsym(libipset_handle,"ipset_load_types");
+	if (!(ipset_session_init_addr = dlsym(libipset_handle, "ipset_session_init")) ||
+	    !(ipset_session_fini_addr = dlsym(libipset_handle, "ipset_session_fini")) ||
+	    !(ipset_session_data_addr = dlsym(libipset_handle,"ipset_session_data")) ||
+	    !(ipset_session_error_addr = dlsym(libipset_handle,"ipset_session_error")) ||
+	    !(ipset_envopt_parse_addr = dlsym(libipset_handle,"ipset_envopt_parse")) ||
+	    !(ipset_type_get_addr = dlsym(libipset_handle,"ipset_type_get")) ||
+	    !(ipset_data_set_addr = dlsym(libipset_handle,"ipset_data_set")) ||
+	    !(ipset_cmd_addr = dlsym(libipset_handle,"ipset_cmd")) ||
+	    !(ipset_load_types_addr = dlsym(libipset_handle,"ipset_load_types"))) {
+		log_message(LOG_INFO, "Failed to dynamic link an ipset function - %s", dlerror());
+		return false;
+	}
+#endif
 
 	ipset_load_types();
 
-	if (!load_mod_xt_set()) {
-		log_message(LOG_INFO, "Unable to load xt_set module");
+	if (!set_match_loaded() && !load_xtables_module("xt_set", "ipsets")) {
+		log_message(LOG_INFO, "Unable to load module xt_set - not using ipsets");
 		return false;
 	}
 
 	return true;
 }
 
-int remove_ipsets(void)
+bool remove_ipsets(void)
 {
 	struct ipset_session *session;
 
+	if (!global_data->using_ipsets)
+		return true;
+
+#ifdef _LIBIPSET_DYNAMIC_
+	if (!libipset_handle)
+		return true;
+#endif
+
 	session = ipset_session_init(printf);
 	if (!session) {
 		log_message(LOG_INFO, "Cannot initialize ipset session.");
 		return false;
 	}
 
-	if (use_ip4tables)
+	if (block_ipv4)
 		ipset_destroy(session, global_data->vrrp_ipset_address);
 
-	if (use_ip6tables) {
+	if (block_ipv6) {
 		ipset_destroy(session, global_data->vrrp_ipset_address6);
 		ipset_destroy(session, global_data->vrrp_ipset_address_iface6);
 	}
@@ -252,33 +305,37 @@ int remove_ipsets(void)
 	return true;
 }
 
-int add_ipsets(bool reload)
+bool add_ipsets(bool reload)
 {
 	return create_sets(global_data->vrrp_ipset_address, global_data->vrrp_ipset_address6, global_data->vrrp_ipset_address_iface6, reload);
 }
 
-struct ipset_session* ipset_session_start(void)
+void* ipset_session_start(void)
 {
 	return ipset_session_init(NULL);
 }
 
-void ipset_session_end(struct ipset_session* session)
+void ipset_session_end(void* vsession)
 {
+	struct ipset_session *session = vsession;
+
 	ipset_session_fini(session);
 }
 
-void ipset_entry(struct ipset_session* session, int cmd, const ip_address_t* addr)
+void ipset_entry(void* vsession, int cmd, const ip_address_t* addr)
 {
 	const char* set;
 	char *iface = NULL;
+	struct ipset_session *session = vsession;
+
 
 	if (addr->ifa.ifa_family == AF_INET) {
-		if (!use_ip4tables)
+		if (!block_ipv4)
 			return;
 		set = global_data->vrrp_ipset_address;
 	}
 	else if (IN6_IS_ADDR_LINKLOCAL(&addr->u.sin6_addr)) {
-		if (!use_ip6tables)
+		if (!block_ipv6)
 			return;
 
 		set = global_data->vrrp_ipset_address_iface6;
