@@ -22,29 +22,28 @@
 
 #include "config.h"
 
-#ifdef _HAVE_LINUX_NET_IF_H_COLLISION_
-/* The following is a horrible workaround. Linux 4.5 introduced a namespace
- * collision when including libiptc/libiptc.h due to both net/if.h and linux/if.h
+#ifdef _HAVE_LIBIPTC_LINUX_NET_IF_H_COLLISION_
+/* Linux 4.5 introduced a namespace collision when including
+ * libiptc/libiptc.h due to both net/if.h and linux/if.h
  * being included.
  *
  * See: http://bugzilla.netfilter.org/show_bug.cgi?id=1067
  *
- * Defining _LINUX_IF_H stops linux/if.h being included.
+ * Including net/if.h first stops the problem occuring.
  */
-#define _LINUX_IF_H
+#include <net/if.h>
 #endif
 
 #include <xtables.h>
 #include <libiptc/libiptc.h>
 #include <libiptc/libip6tc.h>
 #ifdef _HAVE_LIBIPSET_
-#ifdef XT_SET_H_NEEDS_LINUX_IP_SET_H
-#include <libipset/linux_ip_set.h>
-#endif
+#include <linux/netfilter/ipset/ip_set.h>
 #include <linux/netfilter/xt_set.h>
 #endif
 #include <unistd.h>
 #include <signal.h>
+#include <stdint.h>
 
 #include "vrrp_iptables_calls.h"
 #include "memory.h"
@@ -52,6 +51,10 @@
 #if !HAVE_DECL_SOCK_CLOEXEC
 #include "old_socket.h"
 #endif
+#ifdef _LIBIPTC_DYNAMIC_
+#include "global_data.h"
+#endif
+#include "vrrp_iptables.h"
 
 /* We sometimes get a resource_busy on iptc_commit. This appears to happen
  * when someone else is also updating it.
@@ -64,6 +67,63 @@
  * be interferred with by anyone else doing an update.
  */
 
+#ifdef _LIBIPTC_DYNAMIC_
+#include <dlfcn.h>
+
+/* The addresses of the functions we want */
+struct iptc_handle *(*iptc_init_addr)(const char *tablename);
+void (*iptc_free_addr)(struct iptc_handle *h);
+int (*iptc_is_chain_addr)(const char *chain, struct iptc_handle *const handle);
+int (*iptc_insert_entry_addr)(const ipt_chainlabel chain, const struct ipt_entry *e, unsigned int rulenum, struct iptc_handle *handle);
+int (*iptc_append_entry_addr)(const ipt_chainlabel chain, const struct ipt_entry *e, struct iptc_handle *handle);
+int (*iptc_delete_entry_addr)(const ipt_chainlabel chain, const struct ipt_entry *origfw, unsigned char *matchmask, struct iptc_handle *handle);
+int (*iptc_commit_addr)(struct iptc_handle *handle);
+const char *(*iptc_strerror_addr)(int err);
+
+struct ip6tc_handle *(*ip6tc_init_addr)(const char *tablename);
+void (*ip6tc_free_addr)(struct ip6tc_handle *h);
+int (*ip6tc_is_chain_addr)(const char *chain, struct ip6tc_handle *const handle);
+int (*ip6tc_insert_entry_addr)(const ip6t_chainlabel chain, const struct ip6t_entry *e, unsigned int rulenum, struct ip6tc_handle *handle);
+int (*ip6tc_append_entry_addr)(const ip6t_chainlabel chain, const struct ip6t_entry *e, struct ip6tc_handle *handle);
+int (*ip6tc_delete_entry_addr)(const ip6t_chainlabel chain, const struct ip6t_entry *origfw, unsigned char *matchmask, struct ip6tc_handle *handle);
+int (*ip6tc_commit_addr)(struct ip6tc_handle *handle);
+const char *(*ip6tc_strerror_addr)(int err);
+
+/* We can make it look as though normal linking is being used */
+#define iptc_init (*iptc_init_addr)
+#define iptc_free (*iptc_free_addr)
+#define iptc_is_chain (*iptc_is_chain_addr)
+#define iptc_insert_entry (*iptc_insert_entry_addr)
+#define iptc_append_entry (*iptc_append_entry_addr)
+#define iptc_delete_entry (*iptc_delete_entry_addr)
+#define iptc_commit (*iptc_commit_addr)
+#define iptc_strerror (*iptc_strerror_addr)
+
+#define ip6tc_init (*ip6tc_init_addr)
+#define ip6tc_free (*ip6tc_free_addr)
+#define ip6tc_is_chain (*ip6tc_is_chain_addr)
+#define ip6tc_insert_entry (*ip6tc_insert_entry_addr)
+#define ip6tc_append_entry (*ip6tc_append_entry_addr)
+#define ip6tc_delete_entry (*ip6tc_delete_entry_addr)
+#define ip6tc_commit (*ip6tc_commit_addr)
+#define ip6tc_strerror (*ip6tc_strerror_addr)
+
+static void* libip4tc_handle;
+static void* libip6tc_handle;
+#endif
+
+#ifdef _LIBXTABLES_DYNAMIC_
+#include <dlfcn.h>
+
+/* The addresses of the functions we want */
+int (*xtables_insmod_addr)(const char *, const char *, bool);
+
+/* We can make it look as though normal linking is being used */
+#define xtables_insmod (*xtables_insmod_addr)
+
+static void *libxtables_handle;
+#endif
+
 static void
 set_iface(char *vianame, unsigned char *mask, const char *iface)
 {
@@ -198,6 +258,8 @@ int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name,
 
 	sav_errno = errno ;
 
+	FREE(fw);
+
 	if (res !=  1 && (!force || sav_errno != ENOENT))
 	{
 		log_message(LOG_INFO, "ip4tables_process_entry for chain %s returned %d: %s", chain, res, iptc_strerror (sav_errno) ) ;
@@ -326,6 +388,8 @@ int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name
 
 	sav_errno = errno ;
 
+	FREE(fw);
+
 	if (res !=  1 && (!force || sav_errno != ENOENT))
 	{
 		log_message(LOG_INFO, "ip6tables_process_entry for chain %s returned %d: %s", chain, res, ip6tc_strerror (sav_errno) ) ;
@@ -336,12 +400,58 @@ int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name
 	return 0 ;
 }
 
-#ifdef _HAVE_LIBIPSET_
-int load_mod_xt_set(void)
+#ifdef _HAVE_LIBIPTC_
+#ifdef _LIBXTABLES_DYNAMIC_ 
+static
+bool xtables_load(void)
+{
+	if (libxtables_handle)
+		return true;
+
+	if (!(libxtables_handle = dlopen("libxtables.so", RTLD_NOW)) &&
+	    !(libxtables_handle = dlopen(XTABLES_LIB_NAME, RTLD_NOW))) {
+		log_message(LOG_INFO, "Unable to load xtables library - %s", dlerror());
+		return false;
+	}
+
+	if (!(xtables_insmod_addr = dlsym(libxtables_handle, "xtables_insmod"))) {
+		log_message(LOG_INFO, "Failed to dynamic link xtables_insmod - %s", dlerror());
+		dlclose(libxtables_handle);
+		libxtables_handle = NULL;
+
+		return false;
+	}
+
+	return true;
+}
+
+void xtables_unload(void)
+{
+	if (!libxtables_handle)
+		return;
+
+	dlclose(libxtables_handle);
+	libxtables_handle = NULL;
+}
+#endif
+
+bool
+load_xtables_module(const char *module,
+#ifndef _LIBXTABLES_DYNAMIC_
+					__attribute__((unused))
+#endif
+								const char *function)
 {
 	struct sigaction act, old_act;
 	bool res = true;
 
+#ifdef _LIBXTABLES_DYNAMIC_
+	if (!libxtables_handle && !xtables_load()) {
+		log_message(LOG_INFO, "Module %s cannot be loaded; not using %s", module, function);
+		return false;
+	}
+#endif
+
 	/* Enable SIGCHLD since xtables_insmod forks/execs modprobe */
 	act.sa_handler = SIG_DFL;
 	sigemptyset(&act.sa_mask);
@@ -349,13 +459,16 @@ int load_mod_xt_set(void)
 
 	sigaction(SIGCHLD, &act, &old_act);
 
-	if (xtables_insmod("xt_set", NULL, true))
+	if (xtables_insmod(module, NULL, true))
 		res = false;
 
 	sigaction(SIGCHLD, &old_act, NULL);
+
 	return res;
 }
+#endif
 
+#ifdef _HAVE_LIBIPSET_
 #ifndef IP_SET_OP_VERSION	/* Exposed to userspace from Linux 3.4 */
 				/* Copied from <linux/netfilter/ipset/ip_set.h> */
 #define SO_IP_SET	83
@@ -511,7 +624,11 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	struct ipt_entry *fw;
 	struct xt_entry_target *target;
 	struct xt_entry_match *match;
-#ifdef HAVE_XT_SET_INFO_MATCH_V1
+#ifdef HAVE_XT_SET_INFO_MATCH_V4
+	struct xt_set_info_match_v4 *setinfo;
+#elif defined HAVE_XT_SET_INFO_MATCH_V3
+	struct xt_set_info_match_v3 *setinfo;
+#elif defined HAVE_XT_SET_INFO_MATCH_V1
 	struct xt_set_info_match_v1 *setinfo;
 #else
 	struct xt_set_info_match *setinfo;
@@ -539,7 +656,11 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	// set
 	match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
 	match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(*setinfo));
-#ifdef HAVE_XT_SET_INFO_MATCH_V1
+#ifdef HAVE_XT_SET_INFO_MATCH_V4
+	match->u.user.revision = 4;
+#elif defined HAVE_XT_SET_INFO_MATCH_V3
+	match->u.user.revision = 3;
+#elif defined HAVE_XT_SET_INFO_MATCH_V1
 	match->u.user.revision = 1;
 #else
 	match->u.user.revision = 0;
@@ -547,11 +668,17 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 	strcpy(match->u.user.name, "set");
 
-#ifdef HAVE_XT_SET_INFO_MATCH_V1
+#ifdef HAVE_XT_SET_INFO_MATCH_V4
+	setinfo = (struct xt_set_info_match_v4 *)match->data;
+#elif defined HAVE_XT_SET_INFO_MATCH_V3
+	setinfo = (struct xt_set_info_match_v3 *)match->data;
+#elif defined HAVE_XT_SET_INFO_MATCH_V1
 	setinfo = (struct xt_set_info_match_v1 *)match->data;
 #else
 	setinfo = (struct xt_set_info_match *)match->data;
 #endif
+	memset(setinfo, 0, sizeof (*setinfo));
+
 	get_set_byname(set_name, &setinfo->match_set, NFPROTO_IPV4, ignore_errors);
 	if (setinfo->match_set.index == IPSET_INVALID_ID) {
 		FREE(fw);
@@ -622,7 +749,11 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 	struct ip6t_entry *fw;
 	struct xt_entry_target *target;
 	struct xt_entry_match *match;
-#ifdef HAVE_XT_SET_INFO_MATCH_V1
+#ifdef HAVE_XT_SET_INFO_MATCH_V4
+	struct xt_set_info_match_v4 *setinfo;
+#elif defined HAVE_XT_SET_INFO_MATCH_V3
+	struct xt_set_info_match_v3 *setinfo;
+#elif defined HAVE_XT_SET_INFO_MATCH_V1
 	struct xt_set_info_match_v1 *setinfo;
 #else
 	struct xt_set_info_match *setinfo;
@@ -650,7 +781,11 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 	// set
 	match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
 	match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(*setinfo));
-#ifdef HAVE_XT_SET_INFO_MATCH_V1
+#ifdef HAVE_XT_SET_INFO_MATCH_V4
+	match->u.user.revision = 4;
+#elif defined HAVE_XT_SET_INFO_MATCH_V3
+	match->u.user.revision = 3;
+#elif defined HAVE_XT_SET_INFO_MATCH_V1
 	match->u.user.revision = 1;
 #else
 	match->u.user.revision = 0;
@@ -658,11 +793,17 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 	fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 	strcpy(match->u.user.name, "set");
 
-#ifdef HAVE_XT_SET_INFO_MATCH_V1
+#ifdef HAVE_XT_SET_INFO_MATCH_V4
+	setinfo = (struct xt_set_info_match_v4 *)match->data;
+#elif defined HAVE_XT_SET_INFO_MATCH_V3
+	setinfo = (struct xt_set_info_match_v3 *)match->data;
+#elif defined HAVE_XT_SET_INFO_MATCH_V1
 	setinfo = (struct xt_set_info_match_v1 *)match->data;
 #else
 	setinfo = (struct xt_set_info_match *)match->data;
 #endif
+	memset(setinfo, 0, sizeof(*setinfo));
+
 	get_set_byname (set_name, &setinfo->match_set, NFPROTO_IPV6, ignore_errors);
 	if (setinfo->match_set.index == IPSET_INVALID_ID) {
 		FREE(fw);
@@ -727,3 +868,54 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 	return 0;
 }
 #endif
+
+#ifdef _LIBIPTC_DYNAMIC_
+bool iptables_lib_init(void)
+{
+	if (!libip4tc_handle && block_ipv4) {
+		/* Attempt to open the ip4tc library */
+		if (!(libip4tc_handle = dlopen("libip4tc.so", RTLD_NOW)) &&
+		    !(libip4tc_handle = dlopen(IP4TC_LIB_NAME, RTLD_NOW))) {
+			log_message(LOG_INFO, "Unable to load ip4tc library - %s", dlerror());
+			using_libip4tc = false;
+		}
+		else if (!(iptc_init_addr = dlsym(libip4tc_handle, "iptc_init")) ||
+			 !(iptc_free_addr = dlsym(libip4tc_handle, "iptc_free")) ||
+			 !(iptc_is_chain_addr = dlsym(libip4tc_handle,"iptc_is_chain")) ||
+			 !(iptc_insert_entry_addr = dlsym(libip4tc_handle,"iptc_insert_entry")) ||
+			 !(iptc_append_entry_addr = dlsym(libip4tc_handle,"iptc_append_entry")) ||
+			 !(iptc_delete_entry_addr = dlsym(libip4tc_handle,"iptc_delete_entry")) ||
+			 !(iptc_commit_addr = dlsym(libip4tc_handle,"iptc_commit")) ||
+			 !(iptc_strerror_addr = dlsym(libip4tc_handle,"iptc_strerror"))) {
+			log_message(LOG_INFO, "Failed to dynamic link an iptc function - %s", dlerror());
+			using_libip4tc = false;
+			dlclose(libip4tc_handle);
+			libip4tc_handle = NULL;
+		}
+	}
+
+	if (!libip6tc_handle && block_ipv6) {
+		/* Attempt to open the ip6tc library */
+		if (!(libip6tc_handle = dlopen("libip6tc.so", RTLD_NOW)) &&
+		    !(libip6tc_handle = dlopen(IP6TC_LIB_NAME, RTLD_NOW))) {
+			log_message(LOG_INFO, "Unable to load ip6tc library - %s", dlerror());
+			using_libip6tc = false;
+		}
+		else if (!(ip6tc_init_addr = dlsym(libip6tc_handle, "ip6tc_init")) ||
+			 !(ip6tc_free_addr = dlsym(libip6tc_handle, "ip6tc_free")) ||
+			 !(ip6tc_is_chain_addr = dlsym(libip6tc_handle,"ip6tc_is_chain")) ||
+			 !(ip6tc_insert_entry_addr = dlsym(libip6tc_handle,"ip6tc_insert_entry")) ||
+			 !(ip6tc_append_entry_addr = dlsym(libip6tc_handle,"ip6tc_append_entry")) ||
+			 !(ip6tc_delete_entry_addr = dlsym(libip6tc_handle,"ip6tc_delete_entry")) ||
+			 !(ip6tc_commit_addr = dlsym(libip6tc_handle,"ip6tc_commit")) ||
+			 !(ip6tc_strerror_addr = dlsym(libip6tc_handle,"ip6tc_strerror"))) {
+			log_message(LOG_INFO, "Failed to dynamic link an ip6tc function - %s", dlerror());
+			using_libip6tc = false;
+			dlclose(libip6tc_handle);
+			libip6tc_handle = NULL;
+		}
+	}
+
+	return libip4tc_handle || libip6tc_handle;
+}
+#endif
