@@ -24,6 +24,9 @@
 
 #include "config.h"
 
+/* Global includes */
+#include <stdint.h>
+
 #include "vrrp_parser.h"
 #include "vrrp_data.h"
 #include "vrrp_sync.h"
@@ -45,6 +48,9 @@
 #include "bitops.h"
 #include "notify.h"
 
+static bool script_user_set;
+static bool remove_script;
+
 /* Static addresses handler */
 static void
 static_addresses_handler(__attribute__((unused)) vector_t *strvec)
@@ -117,20 +123,17 @@ vrrp_group_handler(vector_t *strvec)
 static inline notify_script_t*
 set_vrrp_notify_script(vector_t *strvec)
 {
-	notify_script_t *script = notify_script_init(strvec, default_script_uid, default_script_gid);
-
-	if (vector_size(strvec) > 2) {
-		if (set_script_uid_gid(strvec, 2, &script->uid, &script->gid))
-			log_message(LOG_INFO, "Invalid user/group for notify script %s", script->name);
-	}
-
-	return script;
+	return notify_script_init(strvec, "notify", global_data->script_security);
 }
 
 static void
 vrrp_gnotify_backup_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	if (vgroup->script_backup) {
+		log_message(LOG_INFO, "vrrp group %s: notify_backup script already specified - ignoring %s", vgroup->gname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vgroup->script_backup = set_vrrp_notify_script(strvec);
 	vgroup->notify_exec = true;
 }
@@ -138,6 +141,10 @@ static void
 vrrp_gnotify_master_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	if (vgroup->script_master) {
+		log_message(LOG_INFO, "vrrp group %s: notify_master script already specified - ignoring %s", vgroup->gname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vgroup->script_master = set_vrrp_notify_script(strvec);
 	vgroup->notify_exec = true;
 }
@@ -145,6 +152,10 @@ static void
 vrrp_gnotify_fault_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	if (vgroup->script_fault) {
+		log_message(LOG_INFO, "vrrp group %s: notify_fault script already specified - ignoring %s", vgroup->gname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vgroup->script_fault = set_vrrp_notify_script(strvec);
 	vgroup->notify_exec = true;
 }
@@ -152,6 +163,10 @@ static void
 vrrp_gnotify_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	if (vgroup->script) {
+		log_message(LOG_INFO, "vrrp group %s: notify script already specified - ignoring %s", vgroup->gname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vgroup->script = set_vrrp_notify_script(strvec);
 	vgroup->notify_exec = true;
 }
@@ -231,6 +246,22 @@ vrrp_unicast_peer_handler(__attribute__((unused)) vector_t *strvec)
 {
 	alloc_value_block(alloc_vrrp_unicast_peer);
 }
+#ifdef _WITH_UNICAST_CHKSUM_COMPAT_
+static void
+vrrp_unicast_chksum_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+
+	if (vector_size(strvec) >= 2) {
+		if (!strcmp(strvec_slot(strvec, 1), "never"))
+			vrrp->unicast_chksum_compat = CHKSUM_COMPATIBILITY_NEVER;
+		else
+			log_message(LOG_INFO, "(%s): Unknown old_unicast_chksum mode %s - ignoring", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
+	}
+	else
+		vrrp->unicast_chksum_compat = CHKSUM_COMPATIBILITY_CONFIG;
+}
+#endif
 static void
 vrrp_native_ipv6_handler(__attribute__((unused)) vector_t *strvec)
 {
@@ -274,6 +305,10 @@ vrrp_int_handler(vector_t *strvec)
 				    , name, vrrp->iname);
 		return;
 	}
+	else if (vrrp->ifp->hw_type == ARPHRD_LOOPBACK) {
+		log_message(LOG_INFO, "(%s): cannot use a loopback interface (%s) for vrrp - ignoring", vrrp->iname, vrrp->ifp->ifname);
+		vrrp->ifp = NULL;
+	}
 }
 static void
 vrrp_track_int_handler(__attribute__((unused)) vector_t *strvec)
@@ -328,7 +363,6 @@ vrrp_vrid_handler(vector_t *strvec)
 	}
 
 	vrrp->vrid = (uint8_t)vrid;
-	alloc_vrrp_bucket(vrrp);
 }
 static void
 vrrp_prio_handler(vector_t *strvec)
@@ -434,6 +468,10 @@ static void
 vrrp_notify_backup_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vrrp->script_backup) {
+		log_message(LOG_INFO, "(%s): notify_backup script already specified - ignoring %s", vrrp->iname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vrrp->script_backup = set_vrrp_notify_script(strvec);
 	vrrp->notify_exec = true;
 }
@@ -441,6 +479,10 @@ static void
 vrrp_notify_master_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vrrp->script_master) {
+		log_message(LOG_INFO, "(%s): notify_master script already specified - ignoring %s", vrrp->iname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vrrp->script_master = set_vrrp_notify_script(strvec);
 	vrrp->notify_exec = true;
 }
@@ -448,6 +490,10 @@ static void
 vrrp_notify_fault_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vrrp->script_fault) {
+		log_message(LOG_INFO, "(%s): notify_fault script already specified - ignoring %s", vrrp->iname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vrrp->script_fault = set_vrrp_notify_script(strvec);
 	vrrp->notify_exec = true;
 }
@@ -455,6 +501,10 @@ static void
 vrrp_notify_stop_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vrrp->script_stop) {
+		log_message(LOG_INFO, "(%s): notify_stop script already specified - ignoring %s", vrrp->iname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vrrp->script_stop = set_vrrp_notify_script(strvec);
 	vrrp->notify_exec = true;
 }
@@ -462,6 +512,10 @@ static void
 vrrp_notify_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vrrp->script) {
+		log_message(LOG_INFO, "(%s): notify script already specified - ignoring %s", vrrp->iname, FMT_STR_VSLOT(strvec,1));
+		return;
+	}
 	vrrp->script = set_vrrp_notify_script(strvec);
 	vrrp->notify_exec = true;
 }
@@ -557,6 +611,24 @@ vrrp_lower_prio_no_advert_handler(vector_t *strvec)
 	}
 }
 
+static void
+vrrp_higher_prio_send_advert_handler(vector_t *strvec)
+{
+	int res;
+
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vector_size(strvec) >= 2) {
+		res = check_true_false(strvec_slot(strvec, 1));
+		if (res >= 0)
+			vrrp->higher_prio_send_advert = (unsigned)res;
+		else
+			log_message(LOG_INFO, "(%s): invalid higher_prio_send_advert %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
+	} else {
+		/* Defaults to true */
+		vrrp->higher_prio_send_advert = true;
+	}
+}
+
 
 #if defined _WITH_VRRP_AUTH_
 static void
@@ -593,43 +665,7 @@ vrrp_auth_pass_handler(vector_t *strvec)
 static void
 vrrp_vip_handler(__attribute__((unused)) vector_t *strvec)
 {
-	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	char *buf;
-	char *str = NULL;
-	vector_t *vec = NULL;
-	sa_family_t address_family;
-
-	buf = (char *) MALLOC(MAXBUF);
-	while (read_line(buf, MAXBUF)) {
-		address_family = AF_UNSPEC;
-		vec = alloc_strvec(buf);
-		if (vec) {
-			str = strvec_slot(vec, 0);
-			if (!strcmp(str, EOB)) {
-				free_strvec(vec);
-				break;
-			}
-
-			if (vector_size(vec)) {
-				alloc_vrrp_vip(vec);
-				if (!LIST_ISEMPTY(vrrp->vip))
-					address_family = IP_FAMILY((ip_address_t*)LIST_TAIL_DATA(vrrp->vip));
-			}
-
-			if (address_family != AF_UNSPEC) {
-				if (vrrp->family == AF_UNSPEC)
-					vrrp->family = address_family;
-				else if (address_family != vrrp->family) {
-					log_message(LOG_INFO, "(%s): address family must match VRRP instance [%s] - ignoring", vrrp->iname, str);
-					free_list_element(vrrp->vip, vrrp->vip->tail);
-				}
-			}
-
-			free_strvec(vec);
-		}
-		memset(buf, 0, MAXBUF);
-	}
-	FREE(buf);
+	alloc_value_block(alloc_vrrp_vip);
 }
 static void
 vrrp_evip_handler(__attribute__((unused)) vector_t *strvec)
@@ -659,6 +695,8 @@ static void
 vrrp_script_handler(vector_t *strvec)
 {
 	alloc_vrrp_script(strvec_slot(strvec, 1));
+	script_user_set = false;
+	remove_script = false;
 }
 static void
 vrrp_vscript_script_handler(vector_t *strvec)
@@ -710,8 +748,39 @@ static void
 vrrp_vscript_user_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
-	if (set_script_uid_gid(strvec, 1, &vscript->uid, &vscript->gid))
-		log_message(LOG_INFO, "Unable to set uid/gid for script %s", vscript->script);
+	if (set_script_uid_gid(strvec, 1, &vscript->uid, &vscript->gid)) {
+		log_message(LOG_INFO, "Unable to set uid/gid for script %s - disabling", vscript->script);
+		remove_script = true;
+	}
+	else
+		script_user_set = true;
+}
+static void
+vrrp_vscript_end_handler(void)
+{
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+
+	if (!vscript->script) {
+		log_message(LOG_INFO, "No script set for vrrp_script %s - removing", vscript->sname);
+		remove_script = true;
+	}
+	else if (!remove_script) {
+		if (script_user_set)
+			return;
+
+		if (set_default_script_user(NULL, NULL, global_data->script_security)) {
+			log_message(LOG_INFO, "Unable to set default user for vrrp script %s - removing", vscript->sname);
+			remove_script = true;
+		}
+	}
+
+	if (remove_script) {
+		free_list_element(vrrp_data->vrrp_script, vrrp_data->vrrp_script->tail);
+		return;
+	}
+
+	vscript->uid = default_script_uid;
+	vscript->gid = default_script_gid;
 }
 static void
 vrrp_vscript_init_fail_handler(__attribute__((unused)) vector_t *strvec)
@@ -883,6 +952,9 @@ init_vrrp_keywords(bool active)
 	install_keyword("vmac_xmit_base", &vrrp_vmac_xmit_base_handler);
 #endif
 	install_keyword("unicast_peer", &vrrp_unicast_peer_handler);
+#ifdef _WITH_UNICAST_CHKSUM_COMPAT_
+	install_keyword("old_unicast_checksum", &vrrp_unicast_chksum_handler);
+#endif
 	install_keyword("native_ipv6", &vrrp_native_ipv6_handler);
 	install_keyword("state", &vrrp_state_handler);
 	install_keyword("interface", &vrrp_int_handler);
@@ -926,6 +998,7 @@ init_vrrp_keywords(bool active)
 	install_keyword("garp_lower_prio_delay", &vrrp_garp_lower_prio_delay_handler);
 	install_keyword("garp_lower_prio_repeat", &vrrp_garp_lower_prio_rep_handler);
 	install_keyword("lower_prio_no_advert", &vrrp_lower_prio_no_advert_handler);
+	install_keyword("higher_prio_send_advert", &vrrp_higher_prio_send_advert_handler);
 #if defined _WITH_VRRP_AUTH_
 	install_keyword("authentication", NULL);
 	install_sublevel();
@@ -942,6 +1015,7 @@ init_vrrp_keywords(bool active)
 	install_keyword("fall", &vrrp_vscript_fall_handler);
 	install_keyword("user", &vrrp_vscript_user_handler);
 	install_keyword("init_fail", &vrrp_vscript_init_fail_handler);
+	install_sublevel_end_handler(&vrrp_vscript_end_handler);
 }
 
 vector_t *
