@@ -23,7 +23,9 @@
 #include "config.h"
 
 #include "vrrp_scheduler.h"
+#ifdef _WITH_VRRP_AUTH_
 #include "vrrp_ipsecah.h"
+#endif
 #include "vrrp_if.h"
 #ifdef _HAVE_VRRP_VMAC_
 #include "vrrp_vmac.h"
@@ -31,13 +33,15 @@
 #include "vrrp.h"
 #include "vrrp_sync.h"
 #include "vrrp_notify.h"
-#include "vrrp_netlink.h"
+#include "keepalived_netlink.h"
 #include "vrrp_data.h"
 #include "vrrp_index.h"
 #include "vrrp_arp.h"
 #include "vrrp_ndisc.h"
 #include "vrrp_if.h"
+#ifdef _WITH_LVS_
 #include "ipvswrapper.h"
+#endif
 #include "memory.h"
 #include "notify.h"
 #include "list.h"
@@ -51,6 +55,7 @@
 #include "vrrp_snmp.h"
 #endif
 #include <netinet/ip.h>
+#include <stdint.h>
 
 /* global vars */
 timeval_t garp_next_time;
@@ -367,9 +372,18 @@ vrrp_compute_timer(const int fd)
 	element e;
 	list l = &vrrp_data->vrrp_index_fd[fd%1024 + 1];
 	timeval_t timer;
+	timer_reset(timer);
+
+	/*
+	 * If list size's is 1 then no collisions. So
+	 * Test and return the singleton.
+	 */
+	if (LIST_SIZE(l) == 1) {
+		vrrp = ELEMENT_DATA(LIST_HEAD(l));
+			return vrrp->sands;
+	}
 
 	/* Multiple instances on the same interface */
-	timer_reset(timer);
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vrrp = ELEMENT_DATA(e);
 		if (timer_cmp(vrrp->sands, timer) < 0 ||
@@ -436,8 +450,9 @@ vrrp_register_workers(list l)
 	vrrp_init_sands(vrrp_data->vrrp);
 
 	/* Init VRRP tracking scripts */
-	if (!LIST_ISEMPTY(vrrp_data->vrrp_script))
+	if (!LIST_ISEMPTY(vrrp_data->vrrp_script)) {
 		vrrp_init_script(vrrp_data->vrrp_script);
+	}
 
 	/* Register VRRP workers threads */
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
@@ -574,6 +589,7 @@ vrrp_set_fds(list l)
 
 				/* append to hash index */
 				alloc_vrrp_fd_bucket(vrrp);
+				alloc_vrrp_bucket(vrrp);
 			}
 		}
 	}
@@ -623,6 +639,7 @@ vrrp_dispatcher_release(vrrp_data_t *data)
 static void
 vrrp_backup(vrrp_t * vrrp, char *buffer, ssize_t len)
 {
+#ifdef _WITH_VRRP_AUTH_
 	struct iphdr *iph;
 	ipsec_ah_t *ah;
 
@@ -635,6 +652,7 @@ vrrp_backup(vrrp_t * vrrp, char *buffer, ssize_t len)
 				vrrp->ipsecah_counter->cycle = 0;
 		}
 	}
+#endif
 
 	if (!VRRP_ISUP(vrrp)) {
 		vrrp_log_int_down(vrrp);
@@ -653,8 +671,13 @@ vrrp_backup(vrrp_t * vrrp, char *buffer, ssize_t len)
 }
 
 static void
-vrrp_become_master(vrrp_t * vrrp, char *buffer, __attribute__((unused)) ssize_t len)
+vrrp_become_master(vrrp_t * vrrp,
+#ifndef _WITH_VRRP_AUTH_
+				 __attribute__((unused))
+#endif
+							 char *buffer, __attribute__((unused)) ssize_t len)
 {
+#ifdef _WITH_VRRP_AUTH_
 	struct iphdr *iph;
 	ipsec_ah_t *ah;
 
@@ -673,6 +696,7 @@ vrrp_become_master(vrrp_t * vrrp, char *buffer, __attribute__((unused)) ssize_t
 			vrrp->ipsecah_counter->cycle = 0;
 		}
 	}
+#endif
 
 	/* Then jump to master state */
 	vrrp->wantstate = VRRP_STATE_MAST;
@@ -869,8 +893,11 @@ vrrp_master(vrrp_t * vrrp)
 
 	/* Then perform the state transition */
 	if (vrrp->wantstate == VRRP_STATE_GOTO_FAULT ||
-	    vrrp->wantstate == VRRP_STATE_BACK ||
-	    (vrrp->version == VRRP_VERSION_2 && vrrp->ipsecah_counter->cycle)) {
+	    vrrp->wantstate == VRRP_STATE_BACK
+#ifdef _WITH_VRRP_AUTH_
+	    || (vrrp->version == VRRP_VERSION_2 && vrrp->ipsecah_counter->cycle)
+#endif
+										) {
 		vrrp->ms_down_timer = 3 * vrrp->adver_int + VRRP_TIMER_SKEW(vrrp);
 
 		/* handle backup state transition */
@@ -1075,6 +1102,22 @@ vrrp_read_dispatcher_thread(thread_t * thread)
 }
 
 /* Script tracking threads */
+bool
+vrrp_child_finder(pid_t pid, char const **name)
+{
+	thread_t *thread;
+
+	for (thread = master->child.head; thread; thread = thread->next)
+	{
+		if (thread->u.c.pid == pid) {
+			vrrp_script_t* scr = THREAD_ARG(thread);
+			*name = scr->script;
+			return false;
+		}
+	}
+	return false;
+}
+
 static int
 vrrp_script_thread(thread_t * thread)
 {
@@ -1113,7 +1156,7 @@ vrrp_script_child_thread(thread_t * thread)
 		vscript->forcing_termination = true;
 		kill(-pid, SIGTERM);
 		thread_add_child(thread->master, vrrp_script_child_timeout_thread,
-				 vscript, pid, 2);
+				 vscript, pid, 2 * 1000000);
 		return 0;
 	}
 
