@@ -51,15 +51,22 @@ enum global_bits {
 	LOG_CONSOLE_BIT,
 	DONT_FORK_BIT,
 	DUMP_CONF_BIT,
+#ifdef _WITH_VRRP_
 	DONT_RELEASE_VRRP_BIT,
+	RELEASE_VIPS_BIT,
+#endif
+#ifdef _WITH_LVS_
 	DONT_RELEASE_IPVS_BIT,
+#endif
 	LOG_DETAIL_BIT,
 	DONT_RESPAWN_BIT,
-	RELEASE_VIPS_BIT,
 	MEM_ERR_DETECT_BIT,
 #ifdef _MEM_CHECK_LOG_
 	MEM_CHECK_LOG_BIT,
 #endif
+#ifdef _WITH_LVS_
+	LOG_ADDRESS_CHANGES,
+#endif
 };
 
 #endif
