@@ -33,7 +33,7 @@
 #define CONTENT_LENGTH	"Content-Length:"
 
 /* Return the http header content length */
-int extract_content_length(char *buffer, int size)
+int extract_content_length(char *buffer, size_t size)
 {
 	char *clen = strstr(buffer, CONTENT_LENGTH);
 
@@ -54,12 +54,13 @@ int extract_content_length(char *buffer, int size)
  * to rfc2616.6.1 status code is between HTTP_Version
  * and Reason_Phrase, separated by space caracter.
  */
-int extract_status_code(char *buffer, int size)
+int extract_status_code(char *buffer, size_t size)
 {
 	char *buf_code;
 	char *begin;
 	char *end = buffer + size;
-	int inc = 0;
+	size_t inc = 0;
+	int code;
 
 	/* Allocate the room */
 	buf_code = (char *)MALLOC(10);
@@ -70,13 +71,13 @@ int extract_status_code(char *buffer, int size)
 	while (buffer < end && *buffer++ != ' ')
 		inc++;
 	strncat(buf_code, begin, inc);
-	inc = atoi(buf_code);
+	code = atoi(buf_code);
 	FREE(buf_code);
-	return inc;
+	return code;
 }
 
 /* simple function returning a pointer to the html buffer begin */
-char *extract_html(char *buffer, int size_buffer)
+char *extract_html(char *buffer, size_t size_buffer)
 {
 	char *end = buffer + size_buffer;
 	char *cur;
