@@ -24,12 +24,14 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE
 #include <glob.h>
 #include <unistd.h>
 #include <libgen.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <fcntl.h>
 #include <stdbool.h>
 
 #include "parser.h"
@@ -53,7 +55,7 @@ static int skip_sublevel = 0;
 
 static char *
 null_strvec(const vector_t *strvec, size_t index)
-{ 
+{
 	if (index - 1 < vector_size(strvec) && index > 0 && vector_slot(strvec, index - 1))
 		log_message(LOG_INFO, "*** Configuration line starting `%s` is missing a parameter after keyword `%s` at word position %zu", vector_slot(strvec, 0) ? (char *)vector_slot(strvec, 0) : "***MISSING ***", (char *)vector_slot(strvec, index - 1), index + 1);
 	else
@@ -62,7 +64,7 @@ null_strvec(const vector_t *strvec, size_t index)
 	exit(KEEPALIVED_EXIT_CONFIG);
 
 	return NULL;
-} 
+}
 
 static void
 keyword_alloc(vector_t *keywords_vec, const char *string, void (*handler) (vector_t *), bool active)
@@ -156,7 +158,6 @@ dump_keywords(vector_t *keydump, int level, FILE *fp)
 	char file_name[21];
 
 	if (!level) {
-		sprintf(file_name, "/tmp/keywords.%d", getpid());
 		snprintf(file_name, sizeof(file_name), "/tmp/keywords.%d", getpid());
 		fp = fopen(file_name, "w");
 		if (!fp)
@@ -263,28 +264,10 @@ process_stream(vector_t *keywords_vec, int need_bob)
 	vector_t *prev_keywords = current_keywords;
 	current_keywords = keywords_vec;
 	int bob_needed = 0;
-	size_t config_id_len = 0;
-	char *buf_start;
-
-	if (config_id)
-		config_id_len = strlen(config_id);
 
 	buf = MALLOC(MAXBUF);
 	while (read_line(buf, MAXBUF)) {
-		if (buf[0] == '@') {
-			/* If the line starts '@', check the following word matches the system id */
-			if (!config_id)
-				continue;
-			buf_start = strpbrk(buf, " \t");
-			if ((size_t)(buf_start - (buf + 1)) != config_id_len ||
-			    strncmp(buf + 1, config_id, config_id_len))
-				continue;
-		}
-		else
-			buf_start = buf;
-
-		strvec = alloc_strvec(buf_start);
-		memset(buf, 0, MAXBUF);
+		strvec = alloc_strvec(buf);
 
 		if (!strvec)
 			continue;
@@ -389,26 +372,37 @@ process_stream(vector_t *keywords_vec, int need_bob)
 	return;
 }
 
-static void
+static bool
 read_conf_file(const char *conf_file)
 {
 	FILE *stream;
-	char *path;
-	int ret;
 	glob_t globbuf;
 	size_t i;
 	int	res;
 	struct stat stb;
+	unsigned num_matches = 0;
 
 	globbuf.gl_offs = 0;
-	res = glob(conf_file, 0, NULL, &globbuf);
+	res = glob(conf_file, GLOB_MARK
+#if HAVE_DECL_GLOB_BRACE
+					| GLOB_BRACE
+#endif
+						    , NULL, &globbuf);
 
 	if (res) {
-		log_message(LOG_INFO, "Unable to find config file(s) '%s'.", conf_file);
-		exit(KEEPALIVED_EXIT_CONFIG);
+		if (res == GLOB_NOMATCH)
+			log_message(LOG_INFO, "No config files matched '%s'.", conf_file);
+		else
+			log_message(LOG_INFO, "Error reading config file(s): glob(\"%s\") returned %d, skipping.", conf_file, res);
+		return true;
 	}
 
-	for(i = 0; i < globbuf.gl_pathc; i++){
+	for (i = 0; i < globbuf.gl_pathc; i++) {
+		if (globbuf.gl_pathv[i][strlen(globbuf.gl_pathv[i])-1] == '/') {
+			/* This is a directory - so skip */
+			continue;
+		}
+
 		log_message(LOG_INFO, "Opening file '%s'.", globbuf.gl_pathv[i]);
 		stream = fopen(globbuf.gl_pathv[i], "r");
 		if (!stream) {
@@ -426,34 +420,38 @@ read_conf_file(const char *conf_file)
 			continue;
 		}
 
+		num_matches++;
+
 		current_stream = stream;
 
-		char prev_path[PATH_MAX];
-		path = getcwd(prev_path, PATH_MAX);
-		if (!path) {
-			log_message(LOG_INFO, "getcwd(%s) error (%s)"
-					    , prev_path, strerror(errno));
+		int curdir_fd = -1;
+		if (strchr(globbuf.gl_pathv[i], '/')) {
+			/* If the filename contains a directory element, change to that directory */
+			curdir_fd = open(".", O_RDONLY | O_DIRECTORY | O_PATH);
+
+			char *confpath = strdup(globbuf.gl_pathv[i]);
+			dirname(confpath);
+			if (chdir(confpath) < 0)
+				log_message(LOG_INFO, "chdir(%s) error (%s)", confpath, strerror(errno));
+			free(confpath);
 		}
 
-		char *confpath = strdup(globbuf.gl_pathv[i]);
-		dirname(confpath);
-		ret = chdir(confpath);
-		if (ret < 0) {
-			log_message(LOG_INFO, "chdir(%s) error (%s)"
-					    , confpath, strerror(errno));
-		}
-		free(confpath);
 		process_stream(current_keywords, 0);
 		fclose(stream);
 
-		ret = chdir(prev_path);
-		if (ret < 0) {
-			log_message(LOG_INFO, "chdir(%s) error (%s)"
-					    , prev_path, strerror(errno));
+		/* If we changed directory, restore the previous directory */
+		if (curdir_fd != -1) {
+			fchdir(curdir_fd);
+			close(curdir_fd);
 		}
 	}
 
 	globfree(&globbuf);
+
+	if (!num_matches)
+		log_message(LOG_INFO, "No config files matched '%s'.", conf_file);
+
+	return false;
 }
 
 bool check_conf_file(const char *conf_file)
@@ -463,33 +461,49 @@ bool check_conf_file(const char *conf_file)
 	bool ret = true;
 	int res;
 	struct stat stb;
+	unsigned num_matches = 0;
 
 	globbuf.gl_offs = 0;
-	res = glob(conf_file, 0, NULL, &globbuf);
+	res = glob(conf_file, GLOB_MARK
+#if HAVE_DECL_GLOB_BRACE
+					| GLOB_BRACE
+#endif
+						    , NULL, &globbuf);
 	if (res) {
 		log_message(LOG_INFO, "Unable to find configuration file %s (glob returned %d)", conf_file, res);
 		return false;
 	}
 
-	if (globbuf.gl_pathc == 0) {
-		log_message(LOG_INFO, "Unable to find configuration file %s", conf_file);
-		ret = false;
-	} else {
-		for (i = 0; i < globbuf.gl_pathc; i++) {
-			if (access(globbuf.gl_pathv[i], R_OK)) {
-				log_message(LOG_INFO, "Unable to read configuration file %s", globbuf.gl_pathv[i]);
-				ret = false;
-				break;
-			}
+	for (i = 0; i < globbuf.gl_pathc; i++) {
+		if (globbuf.gl_pathv[i][strlen(globbuf.gl_pathv[i])-1] == '/') {
+			/* This is a directory - so skip */
+			continue;
+		}
 
-			/* Make sure that the file is a regular file, and not for example a directory or executable */
-			if (stat(globbuf.gl_pathv[i], &stb) ||
-			    !S_ISREG(stb.st_mode) ||
-			     (stb.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))) {
-				log_message(LOG_INFO, "Configuration file '%s' is not a regular non-executable file", globbuf.gl_pathv[i]);
-				ret = false;
-				break;
-			}
+		if (access(globbuf.gl_pathv[i], R_OK)) {
+			log_message(LOG_INFO, "Unable to read configuration file %s", globbuf.gl_pathv[i]);
+			ret = false;
+			break;
+		}
+
+		/* Make sure that the file is a regular file, and not for example a directory or executable */
+		if (stat(globbuf.gl_pathv[i], &stb) ||
+		    !S_ISREG(stb.st_mode) ||
+		     (stb.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))) {
+			log_message(LOG_INFO, "Configuration file '%s' is not a regular non-executable file", globbuf.gl_pathv[i]);
+			ret = false;
+			break;
+		}
+
+		num_matches++;
+	}
+
+	if (ret) {
+		if (num_matches > 1)
+			log_message(LOG_INFO, "WARNING, more than one file matches configuration file %s, using %s", conf_file, globbuf.gl_pathv[0]);
+		else if (num_matches == 0) {
+			log_message(LOG_INFO, "Unable to find configuration file %s", conf_file);
+			ret = false;
 		}
 	}
 
@@ -528,7 +542,13 @@ read_line(char *buf, size_t size)
 {
 	size_t len ;
 	bool eof = false;
+	size_t config_id_len;
+	char *buf_start;
+	bool rev_cmp;
+	size_t ofs;
+	char *text_start;
 
+	config_id_len = config_id ? strlen(config_id) : 0;
 	do {
 		if (fgets(buf, (int)size, current_stream)) {
 			len = strlen(buf);
@@ -536,6 +556,35 @@ read_line(char *buf, size_t size)
 				buf[len-1] = '\0';
 			if (len > 1 && (buf[len-2] == '\n' || buf[len-2] == '\r'))
 				buf[len-2] = '\0';
+			text_start = buf + strspn(buf, " \t");
+			if (text_start[0] == '@') {
+				/* If the line starts '@', check the following word matches the system id.
+				   @^ reverses the sense of the match */
+				if (text_start[1] == '^') {
+					rev_cmp = true;
+					ofs = 2;
+				} else {
+					rev_cmp = false;
+					ofs = 1;
+				}
+
+				/* We need something after the system_id */
+				if (!(buf_start = strpbrk(text_start + ofs, " \t"))) {
+					buf[0] = '\0';
+					break;
+				}
+
+				/* Check if config_id matches/doesn't match as appropriate */
+				if ((!config_id ||
+				     (size_t)(buf_start - (text_start + ofs)) != config_id_len ||
+				     strncmp(text_start + ofs, config_id, config_id_len)) != rev_cmp) {
+					buf[0] = '\0';
+					break;
+				}
+
+				/* Remove the @config_id from start of line */
+				memset(text_start, ' ', (size_t)(buf_start - text_start));
+			}
 		}
 		else
 		{
@@ -543,7 +592,7 @@ read_line(char *buf, size_t size)
 			buf[0] = '\0';
 			break;
 		}
-	} while (check_include(buf));
+	} while (buf[0] && check_include(buf));
 
 	return !eof;
 }
@@ -629,7 +678,6 @@ alloc_value_block(void (*alloc_func) (vector_t *))
 
 			free_strvec(vec);
 		}
-		memset(buf, 0, MAXBUF);
 	}
 	FREE(buf);
 }
@@ -656,6 +704,18 @@ set_value(vector_t *strvec)
 	return alloc;
 }
 
+unsigned long
+read_timer(vector_t *strvec)
+{
+	unsigned long timer;
+
+	timer = strtoul(strvec_slot(strvec, 1), NULL, 10);
+	if (timer >= ULONG_MAX / TIMER_HZ)
+		return ULONG_MAX;
+
+	return timer * TIMER_HZ;
+}
+
 /* Checks for on/true/yes or off/false/no */
 int
 check_true_false(char *str)
