@@ -25,9 +25,11 @@
 #define _SIGNALS_H
 
 /* Prototypes */
+extern int get_signum(const char *);
 extern void *signal_set(int signo, void (*func) (void *, int), void *);
 extern void *signal_ignore(int signo);
-extern void signal_handler_init(int);
+extern void signal_handler_init(void);
+extern void signal_handler_child_clear(void);
 extern void signal_handler_destroy(void);
 extern void signal_handler_script(void);
 extern void signal_run_callback(void);
