@@ -23,12 +23,9 @@
 #include "config.h"
 
 #include <sys/wait.h>
-#include "memory.h"
 #include <unistd.h>
 #include <fcntl.h>
-#include "utils.h"
-#include "signals.h"
-#include "bitops.h"
+#include <stdint.h>
 
 #ifdef _WITH_STACKTRACE_
 #include <sys/types.h>
@@ -37,6 +34,11 @@
 #include <execinfo.h>
 #endif
 
+#include "memory.h"
+#include "utils.h"
+#include "signals.h"
+#include "bitops.h"
+
 /* global vars */
 unsigned long debug = 0;
 
@@ -313,6 +315,8 @@ inet_sockaddrport(struct sockaddr_storage *addr)
 		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
 		port = addr6->sin6_port;
 	} else {
+		/* Note: this might be AF_UNSPEC if it is the sequence number of
+		 * a virtual server in a virtual server group */
 		struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
 		port = addr4->sin_port;
 	}
@@ -323,8 +327,8 @@ inet_sockaddrport(struct sockaddr_storage *addr)
 char *
 inet_sockaddrtopair(struct sockaddr_storage *addr)
 {
-	static char addr_str[INET6_ADDRSTRLEN + 1];
-	static char ret[sizeof(addr_str) + 16];
+	char addr_str[INET6_ADDRSTRLEN];
+	static char ret[sizeof(addr_str) + 8];	/* '[' + addr_str + ']' + ':' + 'nnnnn' */
 
 	inet_sockaddrtos2(addr, addr_str);
 	snprintf(ret, sizeof(ret) - 1, "[%s]:%d"
@@ -333,6 +337,19 @@ inet_sockaddrtopair(struct sockaddr_storage *addr)
 	return ret;
 }
 
+char *
+inet_sockaddrtotrio(struct sockaddr_storage *addr, uint16_t proto)
+{
+	char addr_str[INET6_ADDRSTRLEN];
+	static char ret[sizeof(addr_str) + 13];	/* '[' + addr_str + ']' + ':' + 'sctp' + ':' + 'nnnnn' */
+	char *proto_str = proto == IPPROTO_TCP ? "tcp" : proto == IPPROTO_UDP ? "udp" : proto == IPPROTO_SCTP ? "sctp" : proto == 0 ? "none" : "?";
+
+	inet_sockaddrtos2(addr, addr_str);
+	snprintf(ret, sizeof(ret) - 1, "[%s]:%s:%d" ,addr_str, proto_str,
+		 ntohs(inet_sockaddrport(addr)));
+	return ret;
+}
+
 uint32_t
 inet_sockaddrip4(struct sockaddr_storage *addr)
 {
@@ -535,7 +552,7 @@ set_std_fd(int force)
 	signal_pipe_close(STDERR_FILENO+1);
 }
 
-#ifndef _HAVE_LIBIPTC_
+#if !defined _HAVE_LIBIPTC_ || defined _LIBIPTC_DYNAMIC_
 int
 fork_exec(char **argv)
 {
